
""" Python equivalent of akai section keygrouptools

Keygroup manipulation
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Keygrouptools:
    def __init__(self, z48):
        self.sampler = z48
        self.set_curr_keygroup_cmd = Command('_', '\x10\x01', 'keygrouptools', 'set_curr_keygroup', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_curr_keygroup_cmd = Command('_', '\x10\x02', 'keygrouptools', 'get_curr_keygroup', (), None)
        self.get_group_id_cmd = Command('_', '\x13\x01', 'keygrouptools', 'get_group_id', (), None)
        self.get_edit_mode_cmd = Command('_', '\x13\x02', 'keygrouptools', 'get_edit_mode', (), None)
        self.get_low_note_cmd = Command('_', '\x13\x04', 'keygrouptools', 'get_low_note', (), None)
        self.get_high_note_cmd = Command('_', '\x13\x05', 'keygrouptools', 'get_high_note', (), None)
        self.get_mute_group_cmd = Command('_', '\x13\x06', 'keygrouptools', 'get_mute_group', (), None)
        self.get_fx_override_cmd = Command('_', '\x13\x07', 'keygrouptools', 'get_fx_override', (), None)
        self.get_fx_send_level_cmd = Command('_', '\x13\x08', 'keygrouptools', 'get_fx_send_level', (), None)
        self.get_zone_xfade_cmd = Command('_', '\x13\x09', 'keygrouptools', 'get_zone_xfade', (), None)
        self.get_zone_xfade_type_cmd = Command('_', '\x13\x0A', 'keygrouptools', 'get_zone_xfade_type', (), None)
        self.get_polyphony_cmd = Command('_', '\x13\x0E', 'keygrouptools', 'get_polyphony', (), None)
        self.get_zone_xfade_ctrl_no_cmd = Command('_', '\x13\x0F', 'keygrouptools', 'get_zone_xfade_ctrl_no', (), None)
        self.get_tune_cmd = Command('_', '\x13\x10', 'keygrouptools', 'get_tune', (), None)
        self.get_level_cmd = Command('_', '\x13\x11', 'keygrouptools', 'get_level', (), None)
        self.get_play_trigger_cmd = Command('_', '\x13\x18', 'keygrouptools', 'get_play_trigger', (), None)
        self.get_play_trigger_velocity_cmd = Command('_', '\x13\x19', 'keygrouptools', 'get_play_trigger_velocity', (), None)
        self.get_play_toggle_note_cmd = Command('_', '\x13\x1A', 'keygrouptools', 'get_play_toggle_note', (), None)
        self.get_filter_cmd = Command('_', '\x13\x20', 'keygrouptools', 'get_filter', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_filter_cutoff_cmd = Command('_', '\x13\x21', 'keygrouptools', 'get_filter_cutoff', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_filter_resonance_cmd = Command('_', '\x13\x22', 'keygrouptools', 'get_filter_resonance', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_filter_attenuation_cmd = Command('_', '\x13\x23', 'keygrouptools', 'get_filter_attenuation', (), None)
        self.get_envelope_rate1_cmd = Command('_', '\x13\x30', 'keygrouptools', 'get_envelope_rate1', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_envelope_level1_cmd = Command('_', '\x13\x31', 'keygrouptools', 'get_envelope_level1', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_envelope_rate2_cmd = Command('_', '\x13\x32', 'keygrouptools', 'get_envelope_rate2', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_envelope_level2_cmd = Command('_', '\x13\x33', 'keygrouptools', 'get_envelope_level2', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_envelope_rate3_cmd = Command('_', '\x13\x34', 'keygrouptools', 'get_envelope_rate3', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_envelope_level3_cmd = Command('_', '\x13\x35', 'keygrouptools', 'get_envelope_level3', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_envelope_rate4_cmd = Command('_', '\x13\x36', 'keygrouptools', 'get_envelope_rate4', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_envelope_level4_cmd = Command('_', '\x13\x37', 'keygrouptools', 'get_envelope_level4', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_envelope_reference_cmd = Command('_', '\x13\x42', 'keygrouptools', 'get_envelope_reference', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_attack_hold_cmd = Command('_', '\x13\x43', 'keygrouptools', 'get_attack_hold', (), None)
        self.get_lfo_rate_cmd = Command('_', '\x13\x50', 'keygrouptools', 'get_lfo_rate', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_lfo_delay_cmd = Command('_', '\x13\x51', 'keygrouptools', 'get_lfo_delay', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_lfo_depth_cmd = Command('_', '\x13\x52', 'keygrouptools', 'get_lfo_depth', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_lfo_waveform_cmd = Command('_', '\x13\x53', 'keygrouptools', 'get_lfo_waveform', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_lfo_phase_cmd = Command('_', '\x13\x54', 'keygrouptools', 'get_lfo_phase', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_lfo_shift_cmd = Command('_', '\x13\x55', 'keygrouptools', 'get_lfo_shift', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_lfo_midi_sync_cmd = Command('_', '\x13\x56', 'keygrouptools', 'get_lfo_midi_sync', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_midi_clock_sync_div_cmd = Command('_', '\x13\x57', 'keygrouptools', 'get_midi_clock_sync_div', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_lfo_retrigger_cmd = Command('_', '\x13\x58', 'keygrouptools', 'get_lfo_retrigger', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_lfo_sync_cmd = Command('_', '\x13\x59', 'keygrouptools', 'get_lfo_sync', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_group_id_cmd = Command('_', '\x12\x01', 'keygrouptools', 'set_group_id', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_edit_mode_cmd = Command('_', '\x12\x02', 'keygrouptools', 'set_edit_mode', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_low_note_cmd = Command('_', '\x12\x04', 'keygrouptools', 'set_low_note', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_high_note_cmd = Command('_', '\x12\x05', 'keygrouptools', 'set_high_note', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_mute_group_cmd = Command('_', '\x12\x06', 'keygrouptools', 'set_mute_group', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_fx_override_cmd = Command('_', '\x12\x07', 'keygrouptools', 'set_fx_override', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_fx_send_level_cmd = Command('_', '\x12\x08', 'keygrouptools', 'set_fx_send_level', (aksy.devices.akai.sysex_types.SWORD,), None)
        self.set_zone_xfade_cmd = Command('_', '\x12\x09', 'keygrouptools', 'set_zone_xfade', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_zone_xfade_type_cmd = Command('_', '\x12\x0A', 'keygrouptools', 'set_zone_xfade_type', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_polyphony_cmd = Command('_', '\x12\x0E', 'keygrouptools', 'set_polyphony', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_zone_xfade_ctrl_no_cmd = Command('_', '\x12\x0F', 'keygrouptools', 'set_zone_xfade_ctrl_no', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_tune_cmd = Command('_', '\x12\x10', 'keygrouptools', 'set_tune', (aksy.devices.akai.sysex_types.SWORD,), None)
        self.set_level_cmd = Command('_', '\x12\x11', 'keygrouptools', 'set_level', (aksy.devices.akai.sysex_types.SWORD,), None)
        self.set_play_trigger_cmd = Command('_', '\x12\x18', 'keygrouptools', 'set_play_trigger', (), None)
        self.set_play_trigger_vel_cmd = Command('_', '\x12\x19', 'keygrouptools', 'set_play_trigger_vel', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_play_toggle_note_cmd = Command('_', '\x12\x1A', 'keygrouptools', 'set_play_toggle_note', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_filter_cmd = Command('_', '\x12\x20', 'keygrouptools', 'set_filter', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_filter_cutoff_cmd = Command('_', '\x12\x21', 'keygrouptools', 'set_filter_cutoff', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_filter_resonance_cmd = Command('_', '\x12\x22', 'keygrouptools', 'set_filter_resonance', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_filter_atten_cmd = Command('_', '\x12\x23', 'keygrouptools', 'set_filter_atten', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_envelope_rate1_cmd = Command('_', '\x12\x30', 'keygrouptools', 'set_envelope_rate1', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_envelope_level1_cmd = Command('_', '\x12\x31', 'keygrouptools', 'set_envelope_level1', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_envelope_rate2_cmd = Command('_', '\x12\x32', 'keygrouptools', 'set_envelope_rate2', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_envelope_level2_cmd = Command('_', '\x12\x33', 'keygrouptools', 'set_envelope_level2', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_envelope_rate3_cmd = Command('_', '\x12\x34', 'keygrouptools', 'set_envelope_rate3', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_envelope_level3_cmd = Command('_', '\x12\x35', 'keygrouptools', 'set_envelope_level3', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_envelope_rate4_cmd = Command('_', '\x12\x36', 'keygrouptools', 'set_envelope_rate4', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_envelope_level4_cmd = Command('_', '\x12\x37', 'keygrouptools', 'set_envelope_level4', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_envelope_reference_cmd = Command('_', '\x12\x42', 'keygrouptools', 'set_envelope_reference', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_attack_hold_cmd = Command('_', '\x12\x43', 'keygrouptools', 'set_attack_hold', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_lfo_rate_cmd = Command('_', '\x12\x50', 'keygrouptools', 'set_lfo_rate', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_lfo_delay_cmd = Command('_', '\x12\x51', 'keygrouptools', 'set_lfo_delay', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_lfo_depth_cmd = Command('_', '\x12\x52', 'keygrouptools', 'set_lfo_depth', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_lfo_waveform_cmd = Command('_', '\x12\x53', 'keygrouptools', 'set_lfo_waveform', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_lfo_phase_cmd = Command('_', '\x12\x54', 'keygrouptools', 'set_lfo_phase', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.WORD), None)
        self.set_lfo_shift_cmd = Command('_', '\x12\x55', 'keygrouptools', 'set_lfo_shift', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SBYTE), None)
        self.set_lfo_midi_sync_cmd = Command('_', '\x12\x56', 'keygrouptools', 'set_lfo_midi_sync', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), None)
        self.set_midi_clock_sync_div_cmd = Command('_', '\x12\x57', 'keygrouptools', 'set_midi_clock_sync_div', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_lfo_retrigger_cmd = Command('_', '\x12\x58', 'keygrouptools', 'set_lfo_retrigger', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.WORD), None)
        self.set_lfo_sync_cmd = Command('_', '\x12\x59', 'keygrouptools', 'set_lfo_sync', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)

    def set_curr_keygroup(self, arg0):
        """Select Keygroup to be current <Data1> = Keygroup number (starting with 0)
        """
        return self.sampler.execute(self.set_curr_keygroup_cmd, (arg0, ))

    def get_curr_keygroup(self):
        """Get Current Keygroup

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_curr_keygroup_cmd, ())

    def get_group_id(self):
        """Get Group ID

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_group_id_cmd, ())

    def get_edit_mode(self):
        """Get Keygroup Edit Mode <Reply1> = (0=SINGLE, 1=ALL, 2=ADD)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_edit_mode_cmd, ())

    def get_low_note(self):
        """Get Low Note

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_low_note_cmd, ())

    def get_high_note(self):
        """Get High Note

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_high_note_cmd, ())

    def get_mute_group(self):
        """Get Mute Group <Reply1> = (0=OFF, 1-64=value)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_mute_group_cmd, ())

    def get_fx_override(self):
        """Get FX override <Reply1> = (0=OFF, 1=A, 2=B, 3=C, 4=D, 5=AB, 6=CD, 7=MULTI)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_fx_override_cmd, ())

    def get_fx_send_level(self):
        """Get FX Send Level <Reply1> = level in 10 dB (-600 +60)

        Returns:
            SWORD
        """
        return self.sampler.execute(self.get_fx_send_level_cmd, ())

    def get_zone_xfade(self):
        """Get Zone Crossfade <Reply1> = (0=OFF, 1=VELOCITY, 2=REAL-TIME)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_zone_xfade_cmd, ())

    def get_zone_xfade_type(self):
        """Get Crossfade type <Reply1> = (0=LIN, 1=EXP, 2=LOG)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_zone_xfade_type_cmd, ())

    def get_polyphony(self):
        """Get Polyphony

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_polyphony_cmd, ())

    def get_zone_xfade_ctrl_no(self):
        """Get Zone Crossfade Source Controller Number (only used when Zone Crossfade Source is MIDI CTRL)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_zone_xfade_ctrl_no_cmd, ())

    def get_tune(self):
        """Get Cents Tune

        Returns:
            SWORD
        """
        return self.sampler.execute(self.get_tune_cmd, ())

    def get_level(self):
        """Get Keygroup Level (RT) <Reply> = value in 10 dB

        Returns:
            SWORD
        """
        return self.sampler.execute(self.get_level_cmd, ())

    def get_play_trigger(self):
        """Get Play Trigger <Reply> = (0=NOTE ON, 1=NOTE OFF)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_play_trigger_cmd, ())

    def get_play_trigger_velocity(self):
        """Get Play Trigger Velocity (0 129) <Reply> = (0=ON VEL, 1=OFF VEL, 2-129=0-127)

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_play_trigger_velocity_cmd, ())

    def get_play_toggle_note(self):
        """Get Play Toggle Note <Reply> = (0=OFF, 1=ON)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_play_toggle_note_cmd, ())

    def get_filter(self, arg0):
        """Get Filter <Data1> = Filter block

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_filter_cmd, (arg0, ))

    def get_filter_cutoff(self, arg0):
        """Get Filter Cutoff Frequency. Data1= filter 0-3 reply: (0-100)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_filter_cutoff_cmd, (arg0, ))

    def get_filter_resonance(self, arg0):
        """Get Filter Resonance (0-3) (0-60)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_filter_resonance_cmd, (arg0, ))

    def get_filter_attenuation(self):
        """Get Filter Attenuation (one setting for all filters) <Reply> = (0, 1, 2, 3, 4, 5 = 0dB, 6dB, 12dB, 18dB, 24dB, 30dB)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_filter_attenuation_cmd, ())

    def get_envelope_rate1(self, arg0):
        """Get Envelope Rate1 <Data1> = Envelope (0=AMP, 1=FILTER, 2=AUX) 0-100) Get Envelope Rate 1 (for AMP = Attack)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_rate1_cmd, (arg0, ))

    def get_envelope_level1(self, arg0):
        """Get Envelope Level 1 (FILTER and AUX only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_level1_cmd, (arg0, ))

    def get_envelope_rate2(self, arg0):
        """Get Envelope Rate 2 (for AMP = Decay)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_rate2_cmd, (arg0, ))

    def get_envelope_level2(self, arg0):
        """Get Envelope Level 2 (for AMP = Sustain)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_level2_cmd, (arg0, ))

    def get_envelope_rate3(self, arg0):
        """Get Envelope Rate 3 (for AMP = Release)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_rate3_cmd, (arg0, ))

    def get_envelope_level3(self, arg0):
        """Get Envelope Level 3 (FILTER and AUX only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_level3_cmd, (arg0, ))

    def get_envelope_rate4(self, arg0):
        """Get Envelope Rate 4 (FILTER and AUX only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_rate4_cmd, (arg0, ))

    def get_envelope_level4(self, arg0):
        """Get Envelope Level 4 (FILTER and AUX only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_level4_cmd, (arg0, ))

    def get_envelope_reference(self, arg0):
        """Get Envelope Reference (FILTER and AUX only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_envelope_reference_cmd, (arg0, ))

    def get_attack_hold(self):
        """Get Attack Hold <Reply> = (0=OFF, 1=ON) (AMP only)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_attack_hold_cmd, ())

    def get_lfo_rate(self, arg0):
        """Get LFO Rate (0=LFO1, 1=LFO2)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_lfo_rate_cmd, (arg0, ))

    def get_lfo_delay(self, arg0):
        """Get LFO Delay (0=LFO1, 1=LFO2)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_lfo_delay_cmd, (arg0, ))

    def get_lfo_depth(self, arg0, arg1):
        """Get LFO Depth (0=LFO1, 1=LFO2)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_lfo_depth_cmd, (arg0, arg1, ))

    def get_lfo_waveform(self, arg0):
        """Get LFO Waveform <Reply> = waveform (0=LFO1, 1=LFO2)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_lfo_waveform_cmd, (arg0, ))

    def get_lfo_phase(self, arg0):
        """Get LFO Phase(0=LFO1, 1=LFO2)

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_lfo_phase_cmd, (arg0, ))

    def get_lfo_shift(self, arg0):
        """Get LFO Shift(0=LFO1, 1=LFO2)

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_lfo_shift_cmd, (arg0, ))

    def get_lfo_midi_sync(self, arg0):
        """Get LFO MIDI Sync (0=LFO1, 1=LFO2) <Reply> = (0=OFF, 1=ON)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_lfo_midi_sync_cmd, (arg0, ))

    def get_midi_clock_sync_div(self, arg0):
        """Get MIDI Clock Sync Division <Reply> = value, where 0=8 cy/bt, 1=6cy/bt, 2=4cy/bt, 3=3cy/bt, 4=2cy/bt, 5=1cy/bt 6=2 bt/cy, 7=3bt/cy, ..., 68=64bt/cy

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_midi_clock_sync_div_cmd, (arg0, ))

    def get_lfo_retrigger(self, arg0):
        """Get LFO Re-trigger <Data1>0: LFO1 1: LF02 <Reply> = (0=OFF, 1=ON)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_lfo_retrigger_cmd, (arg0, ))

    def get_lfo_sync(self, arg0):
        """Get LFO sync (i.e., all voices in program locked to same LFO)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_lfo_sync_cmd, (arg0, ))

    def set_group_id(self, arg0):
        """Set Group ID
        """
        return self.sampler.execute(self.set_group_id_cmd, (arg0, ))

    def set_edit_mode(self, arg0):
        """Set Keygroup Edit Mode <Data2> = (0=SINGLE, 1=ALL, 2=ADD)
        """
        return self.sampler.execute(self.set_edit_mode_cmd, (arg0, ))

    def set_low_note(self, arg0):
        """Set Low Note
        """
        return self.sampler.execute(self.set_low_note_cmd, (arg0, ))

    def set_high_note(self, arg0):
        """Set High Note
        """
        return self.sampler.execute(self.set_high_note_cmd, (arg0, ))

    def set_mute_group(self, arg0):
        """Set Mute Group <Data1> = (0=OFF, 1-64=value)
        """
        return self.sampler.execute(self.set_mute_group_cmd, (arg0, ))

    def set_fx_override(self, arg0):
        """Set FX override <Data1> = (0=OFF, 1=A, 2=B, 3=C, 4=D, 5=AB, 6=CD, 7=MULTI)
        """
        return self.sampler.execute(self.set_fx_override_cmd, (arg0, ))

    def set_fx_send_level(self, arg0):
        """Set FX Send Level <Data1> = level in 10 dB (-600 +60)
        """
        return self.sampler.execute(self.set_fx_send_level_cmd, (arg0, ))

    def set_zone_xfade(self, arg0):
        """Set Zone Crossfade <Data1> = (0=OFF, 1=VELOCITY, 2=REAL-TIME)
        """
        return self.sampler.execute(self.set_zone_xfade_cmd, (arg0, ))

    def set_zone_xfade_type(self, arg0):
        """Set Crossfade type <Data1> = (0=LIN, 1=EXP, 2=LOG)
        """
        return self.sampler.execute(self.set_zone_xfade_type_cmd, (arg0, ))

    def set_polyphony(self, arg0):
        """Set Polyphony
        """
        return self.sampler.execute(self.set_polyphony_cmd, (arg0, ))

    def set_zone_xfade_ctrl_no(self, arg0):
        """Set Zone Crossfade Source Controller Number (only used when Zone Crossfade Source is MIDI CTRL)
        """
        return self.sampler.execute(self.set_zone_xfade_ctrl_no_cmd, (arg0, ))

    def set_tune(self, arg0):
        """Set Cents Tune (0 +- 36)
        """
        return self.sampler.execute(self.set_tune_cmd, (arg0, ))

    def set_level(self, arg0):
        """Set Keygroup Level <Data1> = value in 10 dB
        """
        return self.sampler.execute(self.set_level_cmd, (arg0, ))

    def set_play_trigger(self):
        """Set Play Trigger <Data1> = (0=NOTE ON, 1=NOTE OFF)
        """
        return self.sampler.execute(self.set_play_trigger_cmd, ())

    def set_play_trigger_vel(self, arg0):
        """Set Play Trigger Velocity <Data1> = (0=ON VEL, 1=OFF VEL, 2-129=0-127)
        """
        return self.sampler.execute(self.set_play_trigger_vel_cmd, (arg0, ))

    def set_play_toggle_note(self, arg0):
        """Set Play Toggle Note <Data1> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_play_toggle_note_cmd, (arg0, ))

    def set_filter(self, arg0, arg1):
        """Set Filter Mode (0=NORMAL, 1=TRIPLE(1), 2=TRIPLE(2), 3=TRIPLE(3)), type
        """
        return self.sampler.execute(self.set_filter_cmd, (arg0, arg1, ))

    def set_filter_cutoff(self, arg0, arg1):
        """Set Filter Cutoff Frequency
        """
        return self.sampler.execute(self.set_filter_cutoff_cmd, (arg0, arg1, ))

    def set_filter_resonance(self, arg0, arg1):
        """Filter Resonance
        """
        return self.sampler.execute(self.set_filter_resonance_cmd, (arg0, arg1, ))

    def set_filter_atten(self, arg0):
        """Set Filter Attenuation (one setting for all filters) <Data1> = (0, 1, 2, 3, 4, 5 = 0dB, 6dB, 12dB, 18dB, 24dB, 30dB)
        """
        return self.sampler.execute(self.set_filter_atten_cmd, (arg0, ))

    def set_envelope_rate1(self, arg0, arg1):
        """Set Envelope Rate 1 (for AMP = Attack)(0=AMP, 1=FILTER, 2=AUX)
        """
        return self.sampler.execute(self.set_envelope_rate1_cmd, (arg0, arg1, ))

    def set_envelope_level1(self, arg0, arg1):
        """Set Envelope Level 1 (FILTER and AUX only)
        """
        return self.sampler.execute(self.set_envelope_level1_cmd, (arg0, arg1, ))

    def set_envelope_rate2(self, arg0, arg1):
        """Set Envelope Rate 2 (for AMP = Decay)
        """
        return self.sampler.execute(self.set_envelope_rate2_cmd, (arg0, arg1, ))

    def set_envelope_level2(self, arg0, arg1):
        """Set Envelope Level 2 (for AMP = Sustain)
        """
        return self.sampler.execute(self.set_envelope_level2_cmd, (arg0, arg1, ))

    def set_envelope_rate3(self, arg0, arg1):
        """Set Envelope Rate 3 (for AMP = Release)
        """
        return self.sampler.execute(self.set_envelope_rate3_cmd, (arg0, arg1, ))

    def set_envelope_level3(self, arg0, arg1):
        """Set Envelope Level 3 (FILTER and AUX only)
        """
        return self.sampler.execute(self.set_envelope_level3_cmd, (arg0, arg1, ))

    def set_envelope_rate4(self, arg0, arg1):
        """Set Envelope Rate 4 (FILTER and AUX only)
        """
        return self.sampler.execute(self.set_envelope_rate4_cmd, (arg0, arg1, ))

    def set_envelope_level4(self, arg0, arg1):
        """Set Envelope Level 4 (FILTER and AUX only)
        """
        return self.sampler.execute(self.set_envelope_level4_cmd, (arg0, arg1, ))

    def set_envelope_reference(self, arg0, arg1):
        """Set Envelope Reference (FILTER and AUX only)
        """
        return self.sampler.execute(self.set_envelope_reference_cmd, (arg0, arg1, ))

    def set_attack_hold(self, arg0):
        """Set Attack Hold <Data1> = (0=OFF, 1=ON) (AMP only)
        """
        return self.sampler.execute(self.set_attack_hold_cmd, (arg0, ))

    def set_lfo_rate(self, arg0, arg1):
        """Set LFO Rate (0: LFO1 1: LFO2), rate
        """
        return self.sampler.execute(self.set_lfo_rate_cmd, (arg0, arg1, ))

    def set_lfo_delay(self, arg0, arg1):
        """Set LFO Delay <Data2> = delay
        """
        return self.sampler.execute(self.set_lfo_delay_cmd, (arg0, arg1, ))

    def set_lfo_depth(self, arg0, arg1):
        """Set LFO Depth <Data2> = depth
        """
        return self.sampler.execute(self.set_lfo_depth_cmd, (arg0, arg1, ))

    def set_lfo_waveform(self, arg0, arg1):
        """Set LFO Waveform <Data2> = waveform, (see Table 23)
        """
        return self.sampler.execute(self.set_lfo_waveform_cmd, (arg0, arg1, ))

    def set_lfo_phase(self, arg0, arg1):
        """Set LFO Phase
        """
        return self.sampler.execute(self.set_lfo_phase_cmd, (arg0, arg1, ))

    def set_lfo_shift(self, arg0, arg1):
        """Set LFO Shift
        """
        return self.sampler.execute(self.set_lfo_shift_cmd, (arg0, arg1, ))

    def set_lfo_midi_sync(self, arg0, arg1):
        """Set LFO Midi Sync
        """
        return self.sampler.execute(self.set_lfo_midi_sync_cmd, (arg0, arg1, ))

    def set_midi_clock_sync_div(self, arg0, arg1):
        """Set LFO Midi Clock Sync Division
        """
        return self.sampler.execute(self.set_midi_clock_sync_div_cmd, (arg0, arg1, ))

    def set_lfo_retrigger(self, arg0, arg1):
        """Set LFO Retrigger
        """
        return self.sampler.execute(self.set_lfo_retrigger_cmd, (arg0, arg1, ))

    def set_lfo_sync(self, arg0, arg1):
        """Set LFO Sync
        """
        return self.sampler.execute(self.set_lfo_sync_cmd, (arg0, arg1, ))




#!/usr/bin/python

import socket
from aksyosc.osc import OSCMessage, decodeOSC
from oscoptions import create_option_parser

def snd_recv(cmd):
    m = OSCMessage()
    m.setAddress(cmd)
    s.sendall(m.getBinary())
    print decodeOSC(s.recv(8192))

def show_banner():
    print "Aksyosc\n * Enter an osc address at the prompt,\
 e.g. '/systemtools/get_sampler_name'\n * Use 'quit' to exit"

if __name__ == "__main__":
    parser = create_option_parser()
    options = parser.parse_args()[0]
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect((options.address, options.port))
    show_banner()
    try:
        while 1:
            cmd = raw_input("aksyosc> ")
            if not cmd:
                continue
            if cmd == 'quit':
                break
            snd_recv(cmd)
    finally:
        s.shutdown(1)



import threading

LOCK = threading.Lock()
def transaction(lock=LOCK):
    def decorator(func):
        def wrapper(*args, **kwargs):
            lock.acquire()
            try:
                return func(*args, **kwargs)
            finally:
                lock.release()
                
        return wrapper
    return decorator



from aksyosc.osc import OSCMessage, decodeOSC

import socket, logging

LOG = logging.getLogger("aksy.osc.connector")

class OSCConnector:
    """ Execute commands using OSC
    """
    def __init__(self, host, port, timeout=10.0):
        socket.setdefaulttimeout(timeout)
        self.host = host
        self.port = port

    @staticmethod
    def create_msg(command, args):
        m = OSCMessage()
        m.setAddress("/" + command.section + "/" + command.name)
        for arg in args:
            m.append(arg)
        return m.getBinary()
        
    @staticmethod
    def create_socket():
        return socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    def execute(self, command, args):
        b = OSCConnector.create_msg(command, args)
        s = OSCConnector.create_socket()
        resp = self._sendAndRcv(s, b)
        # HACK: mimic the behaviour of TypedComposite
        if len(resp) == 1 and command.reply_spec is None:
            return resp[0]
            
        return resp

    def _sendAndRcv(self, s, b):
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug("Sending message: %s", repr(b))
        s.sendto(b, (self.host, self.port))

        data = s.recv(16384)
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug("Received message: %s", repr(data))

        resp_msg = decodeOSC(data)
        if resp_msg[0] == '/sampler/error':
            raise Exception("Remote execution failed, Server cause: " + resp_msg[3])
        return resp_msg[2:]
    
    @staticmethod
    def create_alt_req_msg(handle, commands, args, index):
        bundle = OSCMessage()
        bundle.setAddress("")
        bundle.append('#bundle')
        bundle.append(0)
        bundle.append(0)
   
        alt_req = OSCMessage()
        alt_req.setAddress('/altoperations')
        alt_req.append(handle)
        alt_req.append(index)
        bundle.append(alt_req.getBinary(), 'b')

        args = list(args)
        
        for cmd in commands:
            if len(cmd.arg_types) > 0:
                arg = args.pop(0)
            else:
                arg = []
                
            bundle.append(OSCConnector.create_msg(cmd, arg), 'b')
        return bundle.message
    
    def execute_alt_request(self, handle, commands, args, index = None):
        b = OSCConnector.create_alt_req_msg(handle, commands, args, index)
        s = OSCConnector.create_socket()
        return self._sendAndRcv(s, b)
    
    def close(self):
        pass




""" Python equivalent of akai section multifxtools

Multi FX
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Multifxtools:
    def __init__(self, z48):
        self.sampler = z48
        self.is_fxcard_installed_cmd = Command('_', '\x24\x01', 'multifxtools', 'is_fxcard_installed', (), None)
        self.get_no_channels_cmd = Command('_', '\x24\x10', 'multifxtools', 'get_no_channels', (), None)
        self.get_max_modules_cmd = Command('_', '\x24\x11', 'multifxtools', 'get_max_modules', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_no_cmd = Command('_', '\x24\x20', 'multifxtools', 'get_no', (), None)
        self.get_name_cmd = Command('_', '\x24\x21', 'multifxtools', 'get_name', (aksy.devices.akai.sysex_types.WORD,), None)
        self.get_id_cmd = Command('_', '\x24\x22', 'multifxtools', 'get_id', (aksy.devices.akai.sysex_types.WORD,), None)
        self.get_param_index_output_ctrl_cmd = Command('_', '\x24\x24', 'multifxtools', 'get_param_index_output_ctrl', (aksy.devices.akai.sysex_types.WORD,), None)
        self.get_number_of_parameters_cmd = Command('_', '\x24\x30', 'multifxtools', 'get_number_of_parameters', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_parameter_minimum_cmd = Command('_', '\x24\x31', 'multifxtools', 'get_parameter_minimum', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_parameter_maximum_cmd = Command('_', '\x24\x32', 'multifxtools', 'get_parameter_maximum', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_parameter_name_cmd = Command('_', '\x24\x33', 'multifxtools', 'get_parameter_name', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_parameter_units_cmd = Command('_', '\x24\x34', 'multifxtools', 'get_parameter_units', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_parameter_type_cmd = Command('_', '\x24\x35', 'multifxtools', 'get_parameter_type', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_display_template_cmd = Command('_', '\x24\x36', 'multifxtools', 'get_display_template', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_parameter_position_id_cmd = Command('_', '\x24\x38', 'multifxtools', 'get_parameter_position_id', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_number_of_parameter_groups_cmd = Command('_', '\x24\x40', 'multifxtools', 'get_number_of_parameter_groups', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_group_name_cmd = Command('_', '\x24\x41', 'multifxtools', 'get_group_name', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_group_index_of_parameter_cmd = Command('_', '\x24\x42', 'multifxtools', 'get_group_index_of_parameter', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_channel_mute_cmd = Command('_', '\x26\x20', 'multifxtools', 'set_channel_mute', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), None)
        self.set_channel_input_cmd = Command('_', '\x26\x21', 'multifxtools', 'set_channel_input', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_channel_output_cmd = Command('_', '\x26\x22', 'multifxtools', 'set_channel_output', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_effect_by_name_cmd = Command('_', '\x26\x30', 'multifxtools', 'set_effect_by_name', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.STRING), None)
        self.set_effect_by_index_cmd = Command('_', '\x26\x31', 'multifxtools', 'set_effect_by_index', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.enable_fx_module_cmd = Command('_', '\x26\x40', 'multifxtools', 'enable_fx_module', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), None)
        self.set_parameter_value_cmd = Command('_', '\x26\x50', 'multifxtools', 'set_parameter_value', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SDWORD), None)
        self.map_qlink_control_cmd = Command('_', '\x26\x52', 'multifxtools', 'map_qlink_control', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.is_channel_muted_cmd = Command('_', '\x27\x20', 'multifxtools', 'is_channel_muted', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_channel_input_cmd = Command('_', '\x27\x21', 'multifxtools', 'get_channel_input', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_channel_output_cmd = Command('_', '\x27\x22', 'multifxtools', 'get_channel_output', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_by_name_cmd = Command('_', '\x27\x30', 'multifxtools', 'get_by_name', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_by_index_cmd = Command('_', '\x27\x31', 'multifxtools', 'get_by_index', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.is_module_enabled_cmd = Command('_', '\x27\x40', 'multifxtools', 'is_module_enabled', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_param_value_cmd = Command('_', '\x27\x50', 'multifxtools', 'get_param_value', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_param_string_cmd = Command('_', '\x27\x51', 'multifxtools', 'get_param_string', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.get_param_qlinkctrl_cmd = Command('_', '\x27\x52', 'multifxtools', 'get_param_qlinkctrl', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_channel_mute_cmd = Command('_', '\x26\x20', 'multifxtools', 'set_channel_mute', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), None)
        self.set_channel_input_cmd = Command('_', '\x26\x21', 'multifxtools', 'set_channel_input', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_channel_output_cmd = Command('_', '\x26\x22', 'multifxtools', 'set_channel_output', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_fx_by_name_cmd = Command('_', '\x26\x30', 'multifxtools', 'set_fx_by_name', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.STRING), None)
        self.set_fx_by_id_cmd = Command('_', '\x26\x31', 'multifxtools', 'set_fx_by_id', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.WORD), None)
        self.enable_module_cmd = Command('_', '\x26\x40', 'multifxtools', 'enable_module', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), None)
        self.set_param_value_cmd = Command('_', '\x26\x50', 'multifxtools', 'set_param_value', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_param_qlinkctrl_cmd = Command('_', '\x26\x52', 'multifxtools', 'set_param_qlinkctrl', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)

    def is_fxcard_installed(self):
        """Get FX card installed

        Returns:
            BYTE
        """
        return self.sampler.execute(self.is_fxcard_installed_cmd, ())

    def get_no_channels(self):
        """Get Number of FX channels

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_no_channels_cmd, ())

    def get_max_modules(self, arg0):
        """Get Maximum Number of FX modules on given channel <Data1=channel>

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_max_modules_cmd, (arg0, ))

    def get_no(self):
        """Get Number of effects available

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_no_cmd, ())

    def get_name(self, arg0):
        """Get effect name <Data1> = index of effect

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_name_cmd, (arg0, ))

    def get_id(self, arg0):
        """Get Unique ID of effect <Data1> = index of effect

        Returns:
            DWORD
        """
        return self.sampler.execute(self.get_id_cmd, (arg0, ))

    def get_param_index_output_ctrl(self, arg0):
        """Get Parameter Index for Output Control <Data1> = index of effect

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_param_index_output_ctrl_cmd, (arg0, ))

    def get_number_of_parameters(self, arg0, arg1):
        """Get Number of Parameters (channel, module)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_number_of_parameters_cmd, (arg0, arg1, ))

    def get_parameter_minimum(self, arg0, arg1, arg2):
        """Get Parameter Minimum (channel, module, param)

        Returns:
            SDWORD
        """
        return self.sampler.execute(self.get_parameter_minimum_cmd, (arg0, arg1, arg2, ))

    def get_parameter_maximum(self, arg0, arg1, arg2):
        """Get Parameter Maximum (channel, module, param)

        Returns:
            SDWORD
        """
        return self.sampler.execute(self.get_parameter_maximum_cmd, (arg0, arg1, arg2, ))

    def get_parameter_name(self, arg0, arg1, arg2):
        """Get Parameter Name (channel, module, param)

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_parameter_name_cmd, (arg0, arg1, arg2, ))

    def get_parameter_units(self, arg0, arg1, arg2):
        """Get Parameter Units (channel, module, param)

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_parameter_units_cmd, (arg0, arg1, arg2, ))

    def get_parameter_type(self, arg0, arg1, arg2):
        """Get Parameter TYPE (channel, module, param)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_parameter_type_cmd, (arg0, arg1, arg2, ))

    def get_display_template(self, arg0, arg1, arg2):
        """Get Parameter Template (channel, module, param)

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_display_template_cmd, (arg0, arg1, arg2, ))

    def get_parameter_position_id(self, arg0, arg1, arg2):
        """Get Parameter Position ID (channel, module, param)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_parameter_position_id_cmd, (arg0, arg1, arg2, ))

    def get_number_of_parameter_groups(self, arg0, arg1):
        """Get Number of Parameter Groups (channel, module)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_number_of_parameter_groups_cmd, (arg0, arg1, ))

    def get_group_name(self, arg0, arg1, arg2):
        """Get Parameter Group name (channel, module, group index)

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_group_name_cmd, (arg0, arg1, arg2, ))

    def get_group_index_of_parameter(self, arg0, arg1, arg2):
        """Get Parameter Group Index (channel, module, param)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_group_index_of_parameter_cmd, (arg0, arg1, arg2, ))

    def set_channel_mute(self, arg0, arg1):
        """Set Mute (channel, mute status)
        """
        return self.sampler.execute(self.set_channel_mute_cmd, (arg0, arg1, ))

    def set_channel_input(self, arg0, arg1):
        """Set Channel Input (channel, input)
        """
        return self.sampler.execute(self.set_channel_input_cmd, (arg0, arg1, ))

    def set_channel_output(self, arg0, arg1):
        """Set Channel Output (channel, output)
        """
        return self.sampler.execute(self.set_channel_output_cmd, (arg0, arg1, ))

    def set_effect_by_name(self, arg0, arg1, arg2):
        """Set Effect on specified Channel (channel, module, effect)
        """
        return self.sampler.execute(self.set_effect_by_name_cmd, (arg0, arg1, arg2, ))

    def set_effect_by_index(self, arg0, arg1, arg2):
        """Set Effect on specified Channel (channel, module, effect)
        """
        return self.sampler.execute(self.set_effect_by_index_cmd, (arg0, arg1, arg2, ))

    def enable_fx_module(self, arg0, arg1, arg2):
        """Enable FX Module (channel, module, enabled)
        """
        return self.sampler.execute(self.enable_fx_module_cmd, (arg0, arg1, arg2, ))

    def set_parameter_value(self, arg0, arg1, arg2, arg3):
        """Set Parameter value (channel, module, param index, value)
        """
        return self.sampler.execute(self.set_parameter_value_cmd, (arg0, arg1, arg2, arg3, ))

    def map_qlink_control(self, arg0, arg1, arg2, arg3):
        """Maps QLink control to a Parameter (channel, module, param index, qlink no)
        """
        return self.sampler.execute(self.map_qlink_control_cmd, (arg0, arg1, arg2, arg3, ))

    def is_channel_muted(self, arg0):
        """Get Mute Status of Channel <Reply> = (0=ON, 1=MUTE)

        Returns:
            BOOL
        """
        return self.sampler.execute(self.is_channel_muted_cmd, (arg0, ))

    def get_channel_input(self, arg0):
        """Get Channel Input <Reply> = input

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_channel_input_cmd, (arg0, ))

    def get_channel_output(self, arg0):
        """Get Channel Output <Reply> = output

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_channel_output_cmd, (arg0, ))

    def get_by_name(self, arg0, arg1):
        """Get effect in module on given channel (by name) <Data1> = channel; <Data2> = module; <Reply> = effect

        Returns:
            BYTE
            STRING
        """
        return self.sampler.execute(self.get_by_name_cmd, (arg0, arg1, ))

    def get_by_index(self, arg0, arg1):
        """Get effect in module on given channel (by index)<Data1> = channel; <Data2> = module; <Reply> = effect

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_by_index_cmd, (arg0, arg1, ))

    def is_module_enabled(self, arg0, arg1):
        """Get Enabled/Disabled State of FX module <Data1> = channel; <Data2> = module; (0=disabled, 1=enabled)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.is_module_enabled_cmd, (arg0, arg1, ))

    def get_param_value(self, arg0, arg1, arg2):
        """Get parameter value of given module in given channel. <Data1> = channel; <Data2> = module; <Data3> = index of parameter

        Returns:
            SDWORD
        """
        return self.sampler.execute(self.get_param_value_cmd, (arg0, arg1, arg2, ))

    def get_param_string(self, arg0, arg1, arg2):
        """Get parameter string of given module in given channel. <Data1> = channel; <Data2> = module; <Data3> = index of parameter

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_param_string_cmd, (arg0, arg1, arg2, ))

    def get_param_qlinkctrl(self, arg0, arg1, arg2):
        """Get Qlink control used to control the parameter <Data1> = channel; <Data2> = module; <Data3> = index of parameter to set <Reply> = Qlink control (0=NONE, 1-n = Qlink 1-n)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_param_qlinkctrl_cmd, (arg0, arg1, arg2, ))

    def set_channel_mute(self, arg0, arg1):
        """Set Mute Status of Channel <Data2> = (0=ON, 1=MUTE)
        """
        return self.sampler.execute(self.set_channel_mute_cmd, (arg0, arg1, ))

    def set_channel_input(self, arg0, arg1):
        """Set Channel Input <Data2> = input
        """
        return self.sampler.execute(self.set_channel_input_cmd, (arg0, arg1, ))

    def set_channel_output(self, arg0, arg1):
        """Set Channel Output <Data2> = output
        """
        return self.sampler.execute(self.set_channel_output_cmd, (arg0, arg1, ))

    def set_fx_by_name(self, arg0, arg1, arg2):
        """Set effect in module on given channel (by name) <Data1> = channel; <Data2> = module; <Data3> = effect.
        """
        return self.sampler.execute(self.set_fx_by_name_cmd, (arg0, arg1, arg2, ))

    def set_fx_by_id(self, arg0, arg1, arg2):
        """Set effect in module on given channel (by index)<Data1> = channel; <Data2> = module; <Data3> = effect
        """
        return self.sampler.execute(self.set_fx_by_id_cmd, (arg0, arg1, arg2, ))

    def enable_module(self, arg0, arg1, arg2):
        """Set State of FX module. (channel, module, enable
        """
        return self.sampler.execute(self.enable_module_cmd, (arg0, arg1, arg2, ))

    def set_param_value(self, arg0, arg1, arg2, arg3):
        """Set parameter value of given module in given channel. <Data1> = channel; <Data2> = module; <Data3> = index of parameter to set <Data4> = parameter value
        """
        return self.sampler.execute(self.set_param_value_cmd, (arg0, arg1, arg2, arg3, ))

    def set_param_qlinkctrl(self, arg0, arg1, arg2, arg3):
        """Set Qlink control used to control the parameter <Data1> = channel; <Data2> = module; <Data3> = index of parameter to set <Data4> = Qlink control (0=NONE, 1-n = Qlink 1-n)
        """
        return self.sampler.execute(self.set_param_qlinkctrl_cmd, (arg0, arg1, arg2, arg3, ))




# Custom widgets via Cairo: Knob, Level, Range, Keygroup etc.

import math, urllib
from urlparse import urlparse

import gobject, gtk

import hitbox, widget

from aksui.utils import modelutils, midiutils
from aksui.ak import *

class SliderWidget(hitbox.HitBox):
    def __init__(self, min, max, w, h, value_descriptions = None, is_scaled = True, index = 0, value_offset = 0):
        # init sliders
        hitbox.HitBox.__init__(self, min, 0, w, h)
        self.value_offset = value_offset
        self.index = index
        # boundaries
        self.max = max
        self.min = min
        # soft max/min, keep sliders from colliding
        self.softmax = max
        self.softmin = min
        self.is_scaled = is_scaled
        # current position
        self.value = self.min
        self.dragging = False
        self.draggingpoint = None

        # value descriptions
        if value_descriptions:
            self.value_descriptions = value_descriptions
        else:
            self.value_descriptions = None

        self.widget = None
        self.state_type = gtk.STATE_NORMAL
        self.shadow_type = gtk.SHADOW_IN

    def set_value(self, value):
        self.x = self.calc_value(value)

    def get_snap(self, s0x):
        last = 0
        calc_value = self.calc_value(s0x)
        for i in octaves_snap:
            if calc_value > last and calc_value < i:
                dist = [calc_value - last, i - calc_value]
                if dist[0] < dist[1]:
                    s0x = last / self.get_scale()
                else:
                    s0x = i / self.get_scale()
                break
            else:
                last = i
        return s0x

    def calc_value(self, value):
        if self.is_scaled:
            scale = self.get_scale()
            newvalue = int(value * scale)
        else:
            newvalue = value

        if newvalue <= self.softmax and newvalue >= self.softmin:
            return newvalue
        elif newvalue > self.softmax:
            return self.softmax
        elif newvalue < self.softmin:
            return self.softmin

    def draw(self, widget, event):
        cr = self.context = widget.window.cairo_create()
        
        if self.dragging:
            color = "blue"
        else:
            color = "black"

        # draw handle at position
        rect = self.get_widget_rect(widget)

        if rect:
            size = widget.window.get_size()

            rect.y = (size[1] / 2) - rect.height / 2

            style = widget.get_style()
            
            style.paint_box(widget.window, self.state_type, self.shadow_type, rect, widget, "", rect.x, rect.y, rect.width, rect.height)

    def draw_text(self, widget, event):
        cr = self.context = widget.window.cairo_create()
        rect = self.get_widget_rect(widget)
        if rect:
            size = widget.window.get_size()

            if self.index != 2:
                cr.set_font_size(10.0)
                cr.set_source_rgb(0.0, 0.0, 0.0)
                fascent, fdescent, fheight, fadvance, fyadvance = cr.font_extents()
                # sloppy
                cr.move_to((size[0] - 60) + (self.index * 30), ((size[1] / 2) - fdescent + fheight / 2) + 1)

                if self.value_descriptions:
                    text = self.value_descriptions[self.x]
                elif self.units == "db":
                    # hack, there's an existing way to do this, just lazy
                    text = '%(#).2f' % {"#" : ((float(self.x) - float(self.value_offset)) / 10.0)}
                else:
                    text = str(self.x - self.value_offset)
                    
                cr.show_text(text)

class AkKnobWidget(widget.AkWidget):
    def __init__(self, samplerobject = None, samplerobjectattr = None, min = -600, max = 60, interval = 10, units = "db", mod_destination = None):
        widget.AkWidget.__init__(self, samplerobject, samplerobjectattr, interval, units)
        self.connect("value-changed", self.on_value_changed)

        self.set_size_request(35, 35)

        self.max = max
        self.min = min

        self.softmax = max
        self.softmin = min

        self.dragging = False
        self.valuestart = None
        self.pinlevelstart = None
        self.draggingstart = None
        if mod_destination:
            self.mod_destination_index = modulationmatrix.ModulationMatrix.destinations.index(mod_destination)
        else:
            self.mod_destination_index = None

        self.queue_draw()

    def get_current_pin(self, create_new = True):
        if isinstance(self.so, keygroup.Keygroup):
            kg = self.so 
        elif hasattr(self.so, 'keygroup'):
            kg = self.so.keygroup
        else:
            return None
         
        return kg.get_pin_by_source_and_dest(kg.current_mod_source_index, self.mod_destination_index, create_new)

    def on_button_press(self, widget, event):
        if event.type == gtk.gdk.BUTTON_PRESS:
            self.dragging = True
            #print "hello...", self.dragging
            self.draggingstart = event.y
            if self.value is not None: 
                self.valuestart = self.value
            else:
                self.valuestart = 0
                
            pin = self.get_current_pin()
            if pin:
                self.pinlevelstart = pin.level
                
        elif event.type == gtk.gdk._2BUTTON_PRESS:
            ctrl_pressed = event.state & gtk.gdk.CONTROL_MASK
            if not ctrl_pressed:
                changed = self.set_value(0.0)
            if changed:
                self.emit("value-changed")
            else:
                pin = self.get_current_pin()
                if pin:
                    pin.set_value(0)
        self.queue_draw()

    def on_button_release(self, widget, event):
        self.dragging = False
        self.draggingstart = None
        self.queue_draw()

    def on_motion_notify_event(self, widget, event):
        #print "motion...", self, type(self.so), self.dragging
        if self.dragging:
            # udate value
            delta = -(event.y - self.draggingstart)

            ctrl_pressed = event.state & gtk.gdk.CONTROL_MASK
            shift_pressed = event.state & gtk.gdk.SHIFT_MASK

            if not shift_pressed:
                interval = self.interval
            else:
                interval = 1.0

            changed = None

            if not ctrl_pressed:
                changed = self.set_value(self.valuestart + (delta * interval))
                
            if changed:
                self.emit("value-changed")
            else:
                # find pin that matches current 'source' and 'destination'
                pin = self.get_current_pin()
                if pin:
                    pin.set_value(self.pinlevelstart + delta)

        self.queue_draw()

    def on_value_changed(self, widget):
        if self.soattr and self.so:
            self.queue_draw()
            self.so.set(self.soattr, int(self.value))

    def get_pct(self, value, min, max):
        if value < min:
            value = min
        elif value > max:
            value = max
        num = float(value) - float(min)
        range = float(max) - float(min)
        pct = float(num) / float(range)
        return pct

    def get_pctatzero(self):
        num = float(value) - float(min)
        range = float(max) - float(min)
        pct = float(num) / float(range)
        pctatzero = abs(float(min) / float(range))
        return pctatzero

    def draw_value_line(self, cr, x, y, radius, value, min, max):
        # the float thing was a little bit unexpected
        pct = self.get_pct(value, min, max)

        self.do_line(cr, x, y, radius, radius, pct, 2)

    def on_expose(self, widget, event):
        self.value = self.so.attrscache[self.soattr]
        #rect = event.area # only redraw a small portion
        size = widget.window.get_size()
        rect = gtk.gdk.Rectangle(0,0,size[0],size[1])
        return self.draw(widget.window, widget, rect)

    def draw(self, window, widget, area):
        cr = window.cairo_create()
        cr.rectangle(area.x, area.y, area.width, area.height)
        cr.clip()

        rect = area

        """
        rect = widget.get_allocation()
        """

        x = rect.width / 2 + area.x
        y = rect.height / 2 + area.y

        radius = (min(rect.width / 2, rect.height / 2) - 10)

        fg = (self.style.fg[gtk.STATE_NORMAL])
        cr.set_source_rgb(fg.red, fg.green, fg.blue)

        cr.set_line_width(1)
        cr.arc(x, y, radius, 0, 2 * math.pi)

        cr.stroke()

        cr.save()

        self.draw_value_line(cr, x, y, radius, self.value, self.min, self.max)

#        pin = self.get_current_pin(False)
#        if pin and self.value is not None:
#            range = float(self.max) - float(self.min)
#            pinoffset = float(float(pin.level) / 100.0) * float(range)
#            pct = self.get_pct(self.value, self.min, self.max)
#            pinpct = self.get_pct(self.value + pinoffset, self.min, self.max)
#            cr.set_source_rgb(0, 0, 0)
#            self.do_mod_arc(cr, x, y, radius, radius, pct, pinpct)
#            self.draw_value_line(cr, x, y, radius, self.value + pinoffset, self.min, self.max)

        """
        cr.set_source_rgb(0, 0, 0)
        self.do_line(cr, x, y, radius, radius / 2, 0, 0.5, False)
        self.do_line(cr, x, y, radius, radius / 2, 1, 0.5, False)
        cr.set_source_rgb(0, 0, 1)
        self.do_line(cr, x, y, radius, radius / 8, pctatzero, 1.0, False)
        """

        cr.set_font_size(10.0)
        cr.set_source_rgb(fg.red, fg.green, fg.blue)
        if self.dragging:
            text = self.get_format()
        else:
            text = self.so.get_knob(self.soattr)

        is_mod = self.soattr.startswith('MOD_')

        if text and len(text) > 0 and (is_mod and text != "0" or not is_mod):
            xbearing, ybearing, width, height, xadvance, yadvance = cr.text_extents(text)
            cr.move_to(x - width / 2 + xbearing, y + radius - ybearing + 2)
            cr.show_text(text)

        cr.restore()
        return False

    def do_line(self, cr, x, y, radius, radius_inset, pct, lw, from_center = True):
        cr.set_line_width(1)
        ei = (math.pi * (6.0/4.0) * pct) - (math.pi * (6.0/8.0))

        if from_center:
            cr.move_to(x, y)
            cr.line_to(x + radius * math.sin( ei ), y + radius * -math.cos( ei ))
        else:
            cr.move_to(x + radius * math.sin( ei ), y + radius * -math.cos( ei ))
            cr.line_to(x + radius_inset * math.sin(ei), y + radius_inset * -math.cos(ei))

        cr.stroke()

    def do_mod_arc(self, cr, x, y, radius, radius_inset, pct, pinpct):
        cr.set_line_width(0.5)
        pct += 1.0
        pinpct += 1.0
        start = (math.pi * (6.0/4.0) * pct) - (math.pi * (6.0/8.0))
        end = (math.pi * (6.0/4.0) * pinpct) - (math.pi * (6.0/8.0))

        cr.move_to(x, y)
        if start < end:
            cr.arc(x, y, radius, start, end)
        else:
            cr.arc_negative(x, y, radius, start, end)
        cr.set_source_rgb(0, 0, 1)
        cr.fill_preserve()
        cr.set_source_rgb(0, 0, 0)
        cr.stroke()

class LevelKnobWidget(AkKnobWidget):
    def __init__(self, samplerobject = None):
        AkKnobWidget.__init__(self, samplerobject, "level", -600, 60, 10, "db")
        self.interval = 10

    def get_format(self):
        return '%(#).2fdb' % {"#" : self.value / self.interval}

class AkRangeWidget(widget.AkWidget):
    def __init__(self, samplerobject = None, samplerobjectattr = None, min = 0, max = 127, multislider = True, value_offset = 0):
        # init sliders
        widget.AkWidget.__init__(self, samplerobject, samplerobjectattr)

        self.multislider = multislider
        
        self.sliderwidth = 8 
        self.sliderheight = 12
        self.middlesliderheight = 12

        self.set_size_request(240, self.sliderheight * 2)

        self.min = min
        self.max = max

        if self.multislider:
            self.sliders = [
                SliderWidget(self.min, self.max, self.sliderwidth, self.sliderheight, None, True, 0, value_offset),
                SliderWidget(self.min, self.max, self.sliderwidth, self.sliderheight, None, True, 1, value_offset),
                SliderWidget(self.min, self.max, 6, self.middlesliderheight, None, False, 2, value_offset)]
            
            self.sliders[0].x = min 
            self.sliders[1].x = max 
            
            self.sliders[0].shadow_type = gtk.SHADOW_IN 
            self.sliders[1].shadow_type = gtk.SHADOW_IN 
            self.sliders[2].shadow_type = gtk.SHADOW_OUT 
        else:
            self.sliders = [
                SliderWidget(self.min, self.max, self.sliderwidth, self.sliderheight, None, True, 0, value_offset),
                ]
            self.sliders[0].x = min 
            self.sliders[0].shadow_type = gtk.SHADOW_OUT 

    def update_middle_slider(self, widget = None):
        if self.multislider:
            s2 = self.sliders[2]
            s1 = self.sliders[1]
            s0 = self.sliders[0]
    
            if widget:
                s0.widget = widget
                s1.widget = widget
                s2.widget = widget
    
            self.sliders[2].x = int(self.sliders[0].get_scaled_value() + s0.w + 2)
            self.sliders[2].y = 1
            self.sliders[2].w = max(int(self.sliders[1].get_scaled_value() + s1.w + s0.w - self.sliders[2].x),0)
            #print self.sliders[2].x, self.sliders[2].y, self.sliders[2].w
    
    def on_button_press(self, widget, event):
        for slider in self.sliders:
            slider.widget = widget
            if slider.point_in_widget_rect(event.x, event.y): 
                if not slider.dragging:
                    slider.dragging = True
                    slider.draggingpoint = [event.x, event.y]
                    self.sliders[0].draggingvalue = self.sliders[0].x
                    if self.multislider:
                        self.sliders[1].draggingvalue = self.sliders[1].x
                        self.sliders[2].draggingvalue = self.sliders[2].x
                    # mark slider as being dragged
        self.queue_draw()

    def on_button_release(self, widget, event):
        for slider in self.sliders:
            slider.widget = widget
            slider.dragging = False
            slider.draggingpoint = None

        self.queue_draw()

    def on_motion_notify_event(self, widget, event):
        # iterate through sliders
        s0 = self.sliders[0]
        s0xvalue = s0.x
        
        if self.multislider:
            s1 = self.sliders[1]
            s0.softmax = s1.x
            s1.softmin = s0.x
            s1xvalue = s1.x

        ctrl_pressed = event.state & gtk.gdk.CONTROL_MASK

        for slider in self.sliders:
            slider.widget = widget
            if slider.dragging:
                delta = event.x - slider.draggingpoint[0]

                """
                if ctrl_pressed:
                    delta *= 0.25
                """

                if self.multislider and self.sliders.index(slider) == 2:
                        s0x = (s0.draggingvalue / s0.get_scale()) + delta
                        s1x = (s1.draggingvalue / s1.get_scale()) + delta
    
                        if ctrl_pressed:
                            s0x = s0.get_snap(s0x)
                            s1x = s1.get_snap(s1x)
    
                        s0.set_value(s0x)
                        s1.set_value(s1x)
                else:
                    slider.set_value((slider.draggingvalue / slider.get_scale()) + delta)
    
                    slider.state_type = gtk.STATE_ACTIVE
            elif slider.point_in_widget_rect(event.x, event.y):
                slider.state_type = gtk.STATE_PRELIGHT
            else:                
                slider.state_type = gtk.STATE_NORMAL

        self.queue_draw()

        if s0xvalue != s0.x: 
            self.emit("slider_1_changed", s0.x)
            
        if self.multislider:
            if s1xvalue != s1.x:
                self.emit("slider_2_changed", s1.x)

        return True

    def on_expose(self, widget, event):
        self.update_middle_slider(widget)
        self.context = widget.window.cairo_create()

        # draw handles
        for slider in self.sliders:
            slider.draw(widget, event)
        # draw text
        for slider in self.sliders:
            slider.draw_text(widget, event)

        return False

class PartRangeWidget(AkRangeWidget):
    def __init__(self, part, attr):
        AkRangeWidget.__init__(self, part, None, 0, 660, False, 600)
        self.part = part
        self.part.update()
        self.s = part.s
        self.attr = attr
        self.value_offset = 600

        self.sliders[0].x = getattr(self.part, attr) + self.value_offset
        self.sliders[0].units = "db"

        self.connect("slider_1_changed", self.on_slider_1_changed)

        self.queue_draw()

    def on_slider_1_changed(self, widget, value):
        if self.part:
            self.part.set(self.attr, value - self.value_offset)

class KeygroupRangeWidget(AkRangeWidget):
    def __init__(self, keygroup):
        AkRangeWidget.__init__(self, keygroup, None, 0, 127)
        self.keygroup = keygroup
        self.keygroup.update()
        self.s = keygroup.s

        self.sliders[0].value_descriptions = midiutils.midinotes
        self.sliders[1].value_descriptions = midiutils.midinotes
        self.sliders[0].x = self.keygroup.low_note
        self.sliders[1].x = self.keygroup.high_note

        self.connect("slider_1_changed", self.on_slider_1_changed)
        self.connect("slider_2_changed", self.on_slider_2_changed)

        self.queue_draw()

    def on_slider_1_changed(self, widget, value):
        if self.keygroup:
            self.keygroup.set("low_note", value)

    def on_slider_2_changed(self, widget, value):
        if self.keygroup:
            self.keygroup.set("high_note", value)

    @staticmethod
    def test_AkRangeWidget():
        test = tc()

class MiniZoneWidget(widget.AkWidget):
    def __init__(self, zone):
        widget.AkWidget.__init__(self, zone)
        
        self.zone = zone
        self.zone.update()
        self.zone.set_callback = self.on_set_callback

        self.drag_dest_set(0, [], 0)

        self.connect("drag_data_received", self.on_drag_data_received)
        self.connect("drag_motion", self.on_drag_motion)
        self.connect("button_press_event", self.on_button_press)
        self.connect("button_release_event", self.on_button_release)
        self.connect("motion_notify_event", self.on_motion_notify_event)
        self.connect("expose_event", self.on_expose)

        self.add_events(gtk.gdk.BUTTON_PRESS_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK |
                        gtk.gdk.POINTER_MOTION_MASK)

        self.set_size_request(100, 15)
        
    def on_set_callback(self, attrname, attrval):
        self.queue_draw()

    def do_upload(self, filename):
        return True

    def on_drag_motion(self, widget, context, x, y, timestamp):
        #print "huh"
        context.drag_status(gtk.gdk.ACTION_COPY, time)
        l.set_text('\n'.join([str(t) for t in context.targets]))
        return True

    def on_drag_data_received(self, widget, context, x, y, selection, target_type, timestamp):
        #print selection.data, target_type
        if target_type == 0:
            # try uploading that shit
            parsed = urlparse(selection.data.rstrip('\r\n'))
            path = urllib.unquote(parsed[2])
            self.s.filechooser.upload(path)
            samplename = selection.data
            zone.set_sample(samplename)

        context.finish(True, False, timestamp)

        return True

    def on_motion_notify_event(self, widget, event):
        #self.queue_draw()
        return True

    def on_button_press(self, widget, event):
        self.zonewin = gtk.Window()
        self.zoneeditor = UI.ZoneEditor(self.zone)
        self.zonewin.add(self.zoneeditor.editor)
        self.zonewin.show_all()
        self.queue_draw()
        return True

    def on_button_release(self, widget, event):
        return True

    def on_expose(self, widget, event):
        size = widget.window.get_size()
        rect = gtk.gdk.Rectangle(0, 0, size[0], size[1])
        style = widget.get_style()

        if self.zone.sample != "":
            shadow_style = gtk.SHADOW_OUT
        else:
            shadow_style = gtk.SHADOW_IN

        style.paint_box(widget.window, gtk.STATE_NORMAL, shadow_style, rect, widget, "", 0, 0, rect.width, rect.height)

        # paint text 
        
        cr = widget.window.cairo_create()
        cr.set_font_size(8.0)
        cr.set_source_rgb(0.0, 0.0, 0.0)
        fascent, fdescent, fheight, fadvance, fyadvance = cr.font_extents()
        cr.move_to(2, ((size[1] / 2) - fdescent + fheight / 2))
        if self.zone.sample != "":
            cr.show_text(self.zone.sample)
            
        return False

class AkLabel(gtk.Label):
    def __init__(self, so, soattr, model, use_index = True):
        """
        TODO: ADD CONTEXT MENU TO CHANGE SAMPLE
        """
        gtk.Label.__init__(self, "sample")
        self.somodel = model
        self.init(so, soattr)
        
    def init(self, so, soattr):
        self.updating = True
        iter = None

        if soattr:
            self.soattr = soattr
            
        if so:
            self.so = so
        else:
            so = self.so

        if so:
            self.s = so.s
            self.so = so
            self.value = None

            if self.soattr:
                self.value = getattr(so, self.soattr)
                self.set_text(self.value)
                
        self.updating = False
        self.queue_draw()
        
class AkComboBox(gtk.ComboBox):
    def __init__(self, so, soattr, model, use_index = True):
        if type(model) is list:
            model = modelutils.get_model_from_list(model)

        gtk.ComboBox.__init__(self, model)

        #self.set_size_request(50, 25)

        cell = gtk.CellRendererText()
        self.use_index = use_index # use value, versus index
        self.pack_start(cell, False)
        self.add_attribute(cell, 'text', 0)  
        self.connect("changed", self.on_changed)
        self.somodel = model

        self.init(so, soattr)
        
    def init(self, so = None, soattr = None):
        self.updating = True
        iter = None

        if soattr:
            self.soattr = soattr
            
        if so:
            self.so = so
        else:
            so = self.so

        if so:
            self.s = so.s
            self.so = so
            self.value = None

            if self.soattr:
                self.value = getattr(so, self.soattr)
                if self.value != None:
                    if not self.use_index:
                        if len(self.value) > 0:
                            iter = self.find_iter(self.value)
                        if iter:
                            self.set_active_iter(iter)
                        elif len(self.value) > 0:
                            print "missing iter for", self.value, "model probably not initialized?"
                    elif self.use_index:
                        self.set_active(int(self.value))
                    else:
                        self.set_active(-1)

        self.updating = False
        self.queue_draw()

    def find_iter(self, value):
        iter = modelutils.search(self.somodel, self.somodel.iter_children(None), None, (0, value)) 
        return iter

    def get_text(self):
        iter = self.get_active_iter()
        value = self.somodel[iter][0]
        return value 
    
    def on_changed(self, widget):
        if not self.updating:
            active = self.get_active_iter()

            if self.use_index:
                value = self.get_active()
                self.so.set(self.soattr, value)
            else:
                value = self.somodel[active][0]
                self.so.set(self.soattr, value)

            self.value = value

    def on_expose(self, widget):
        return True

gobject.signal_new("slider_1_changed", AkRangeWidget, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_INT,)) 
gobject.signal_new("slider_2_changed", AkRangeWidget, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_INT,)) 
gobject.signal_new("value_changed", AkKnobWidget, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()) 



from time import time

import stat
import os
import os.path
import errno
import logging

from aksy.device import Devices
from aksy import fileutils
from aksy.concurrent import transaction

from aksy.devices.akai import sampler as samplermod
from aksy.devices.akai import model
from aksy.config import get_config

MAX_FILE_SIZE_SAMPLE = 512 * 1024 * 1024 # 512 MB
MAX_FILE_SIZE_OTHER = 16 * 1024 # 16K
START_TIME = time()

LOG = logging.getLogger('aksy.aksyfs.common')

class StatInfo(object):
    @staticmethod
    def set_owner(uid, gid):
        StatInfo.uid = uid
        StatInfo.gid = gid
        
    def __init__(self, mode, size, writable=False):
        self.st_dev = 0 # TODO: figure out whether required to provide unique value
        self.st_ino = 0 # TODO: figure out whether required to provide unique value
        self.st_mode = mode|stat.S_IRUSR|stat.S_IRGRP
        if writable:
            self.st_mode |= stat.S_IWUSR

        self.st_size = size
        self.st_atime = int(time())
        self.st_ctime = self.st_atime
        if is_modified:
            self.st_mtime = int(time())
        else:
            self.st_mtime = self.st_atime
        self.st_uid = StatInfo.uid
        self.st_gid = StatInfo.gid

class DirStatInfo(StatInfo):   
    def __init__(self, writable=False, child_count=1):
        StatInfo.__init__(self, stat.S_IFDIR|stat.S_IEXEC, 4096L)
        self.st_nlink = child_count

class FileStatInfo(StatInfo):
    def __init__(self, path, size, writable=False):
        if size is None:
            size = get_file_size(path)
        StatInfo.__init__(self, stat.S_IFREG|stat.S_IWUSR, size, writable)
        self.st_nlink = 0
        
def is_modified(stat_info):
    return stat_info.st_mtime > START_TIME

def _splitpath(path):
    return path.split('/', 2)[1:]

def _get_cache_path(path):
    return os.path.join(os.path.expanduser('~'), '.aksy/cache' + path)

def get_file_size(path):
    cache_path = _get_cache_path(path)
    if os.path.exists(cache_path):
        return os.lstat(cache_path).st_size
    
    if fileutils.is_sample(path):
        return MAX_FILE_SIZE_SAMPLE
    
    return MAX_FILE_SIZE_OTHER

def _cache_path_exists(path):
    return os.path.exists(_get_cache_path(path))

def _create_cache_path(path):
    cache_path = _get_cache_path(path)
    parent_dir = os.path.dirname(cache_path)
    if not os.path.exists(parent_dir):
        os.makedirs(parent_dir)
    return cache_path

class AksyFile(object):
    @staticmethod
    def set_sampler(sampler):
        AksyFile.sampler = sampler
    
    @staticmethod
    def parse_location(location):
        if location == "memory":
            return samplermod.AkaiSampler.MEMORY
        elif location == "disks":
            return samplermod.AkaiSampler.DISK
        else:
            raiseException(errno.ENOENT)
            
    @transaction()
    def __init__(self, path, flags, *mode):
        self.direct_io = True
        self.upload = bool(flags & os.O_WRONLY)
        self.name = os.path.basename(path)
        self.location = AksyFile.parse_location(_splitpath(path)[0])

        self.path = _create_cache_path(path)

        if not self.is_upload() or not _cache_path_exists(path):
            AksyFile.sampler.get(self.name, self.path, self.location)
        
        self.handle = os.open(self.path, flags, *mode)

    @transaction()
    def release(self, flags):
        if self.is_upload():
            try:
                AksyFile.sampler.transfertools.put(self.get_path(), None, self.get_location())
            except IOError, exc:
                # TODO: move to a method where we can raise exceptions
                LOG.exception( "Exception occurred: %s", repr(exc))
        os.close(self.handle)

    def write(self, buf, offset):
        os.lseek(self.handle, offset, 0)
        return os.write(self.handle, buf)

    def read(self, length, offset):
        os.lseek(self.handle, offset, 0)
        return os.read(self.handle, length)

    def get_name(self):
        return self.name
    
    def get_path(self):
        return self.path
    
    def is_upload(self):
        return self.upload

    def get_handle(self):
        return self.handle
    
    def get_location(self):
        return self.location

class FSRoot(model.Container):
    def __init__(self, sampler):
        self.sampler = sampler
        
    def is_writable(self):
        return False
    
    def get_children(self):
        return [self.sampler.memory, self.sampler.disks]

class AksyFS(object): #IGNORE:R0904
    def __init__(self, sampler=None):
        if sampler is not None:
            self.init_sampler(sampler)
        stat_home = os.stat(os.path.expanduser('~'))
        StatInfo.set_owner(stat_home[stat.ST_UID], stat_home[stat.ST_GID])

    @transaction()
    def open_for_read(self, path, mode):
        location = _splitpath(path)[0]
        name = os.path.basename(path)
        path = _create_cache_path(path)

        if location == "memory":
            location = samplermod.AkaiSampler.MEMORY
        elif location == "disks":
            location = samplermod.AkaiSampler.DISK
        else:
            raiseException(errno.ENOENT)
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug( "open(%s, %s)", path, mode)
        self.sampler.transfertools.get(name, path, location)

        return open(path, mode)

    @transaction()
    def open_for_write(self, path, mode):
        dest = AksyFile.parse_location(_splitpath(path)[0])
        name = os.path.basename(path)
        path = _create_cache_path(path)

        handle = open(path, mode)
        
        class UploadingFileWrapper:
            def __init__(self, fd, sampler):
                self.sampler = sampler
                self.file = fd
                
            def __getattr__(self, attr):
                return getattr(self.file, attr)
            
            def close(self):
                self.file.close()
                self.sampler.transfertools.put(path, name, dest)

        return UploadingFileWrapper(handle, self.sampler)
        
    def fetch_parent_folder(self, path):
        return self.get_parent(path).get_child(os.path.basename(path))

    @transaction()
    def stat_directory(self, path):
        folder = self.cache.get(path)
        if folder is None:
            folder = self.fetch_parent_folder(path)
            if folder is None:
                raiseException(errno.ENOENT)
            self.cache[path] = folder
        return DirStatInfo(folder.is_writable())

    def get_parent(self, path):
        parent = os.path.dirname(path)
        return self.cache[parent]
        
    def get_file(self, path):
        if not samplermod.Sampler.is_filetype_supported(path):
            raiseException(errno.ENOENT)

        parent_dir = self.get_parent(path)
        file_name = os.path.basename(path)
        
        for child in parent_dir.get_children():
            if child.get_name() == file_name:
                return child
        return None

    @transaction()
    def stat_file(self, path):
        cached = self.cache.get(path, None)
        if cached is not None:
            return cached
        if _cache_path_exists(path):
            return os.lstat(_create_cache_path(path))
        
        file_obj = self.get_file(path)
        if file_obj is None:
            raiseException(errno.ENOENT)
        else:
            return FileStatInfo(path, file_obj.get_size(), False)

    def getattr(self, path):
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug( 'getattr(%s)', path)
        if fileutils.is_dirpath(path):
            return self.stat_directory(path)
        else:
            return self.stat_file(path)

    @transaction()
    def listdir(self, path):
        folder = self.cache.get(path, None)
        if folder is None:
            raiseException(errno.ENOENT)
        for child in folder.get_children():
            yield child.get_name()

    @transaction()
    def mkdir(self, path, mode='unused'):
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug('mkdir(%s, %s)', path, mode)
        folder = self.get_parent(path)
        if not folder.is_writable():
            raiseException(errno.EROFS)
        if not hasattr(folder, 'create_folder'):
            raiseUnsupportedOperationException()
        
        child = folder.create_folder(os.path.basename(path))
        self.cache[path] = child

    @transaction()
    def rename(self, old_path, new_path):
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug('rename(%s, %s)', old_path, new_path)
        new_dir = os.path.dirname(new_path)
        if os.path.dirname(old_path) != new_dir:
            file_obj = self.get_file(old_path)
            if hasattr(file_obj, 'save'):
                folder = self.cache[new_dir]
                folder.set_current()
                file_obj.save(False, False)
            elif hasattr(file_obj, 'load'):
                file_obj.load()
            else:
                raiseUnsupportedOperationException() 
            return
                
        new_name = os.path.basename(new_path)
        if fileutils.is_dirpath(old_path):
            folder = self.cache[old_path]
            folder.rename(new_name)
            self.cache[new_path] = folder
            del self.cache[old_path]
        else: 
            file_obj = self.get_file(old_path)
            file_obj.rename(new_name)

    @transaction()
    def rmdir(self, path):
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug('rmdir(%s)', path)
        folder = self.cache[path]
        if hasattr(folder, 'delete'):
            folder.delete()
            del self.cache[path]
            self.get_parent(path).refresh()
            return

        raiseException(errno.EPERM)

    @transaction()
    def unlink(self, path):
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug('unlink(%s)', path)
        file_obj = self.get_file(path)
        file_obj.delete()
        try:
            os.remove(_create_cache_path(path))
        except OSError:
            # file not cached
            pass 
        
        self.get_parent(path).refresh()

    def init_sampler(self, sampler):
        self.root = FSRoot(sampler)
        self.sampler = sampler
        self.cache = { '/': self.root }
        AksyFile.set_sampler(sampler)

def raiseUnsupportedOperationException():
    raiseException(errno.ENOSYS)

def raiseException(err):
    raise OSError(err, 'Exception occurred')



#!/usr/bin/env python

__author__ = """Joseph Misra (main developer)
Walco van Loon (back-end, deployment, refactorings & UI enhancements)
"""
__version__ = '0.3'
__license__ = """
    Copyright (C) 2006-2007  %s

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
""" % __author__

try:
    import hotshot, hotshot.stats
except ImportError:
    print "Profiler not available"

import traceback, thread, os.path

import pygtk
pygtk.require('2.0')
import gtk

# our stuff
from aksui.utils import midiutils, modelutils
from aksui.ak import multi, recording, program, keygroup
from aksui.UI import base, filechooser, multieditor, keygroupeditor, lcdscreen, recorddialog, multifxeditor, programdetails

from aksy.device import Devices
from aksy import config
from aksy.concurrent import transaction



# config
USE_CUSTOM_EXCEPTHOOK = False # this gets in the way of eclipse's handy exception line # link feature, could probably fix later
ENABLE_PROFILER = False
TARGET_TYPE_URI_LIST = 80



def get_selected_from_treeview(treeview):
    """
    will return a single value or a list depending on what the selection mode is
    """
    selection = treeview.get_selection()
    if selection.get_mode() == gtk.SELECTION_MULTIPLE:
        (model, pathlist) = selection.get_selected_rows()
        result = []
        for path in pathlist:
            result.append(model[path][0])
        return result
    else:
        (model, iter) = selection.get_selected()
        return model[iter][0] 

# exception handler ripped from austin's code 
def exceptionHandler(type, value, tback):    
    try:
        print ""
        print "-- Initialization Exception --"
        tbmessage = "Type: " + str(type) + "\nValue: " + str(value) + "\nData:\n"
        tblist = traceback.extract_tb(tback)
        for x in tblist:
            tbmessage = tbmessage + str(x) + "\n"
        print "Error Code:", tbmessage
        
    except:
        traceback.print_exc()

    return 0


class BaseDialog(base.Base):
    def __init__(self, name):
        base.Base.__init__(self, None, name)

    def on_okbutton_clicked(self, widget):
        self.editor.response(gtk.RESPONSE_OK)
        self.editor.hide()

    def on_cancelbutton_clicked(self, widget):
        self.editor.response(gtk.RESPONSE_CANCEL)
        self.editor.hide()

class DialogCreateNewKeygroups(BaseDialog):
    def __init__(self, parent):
        self.s = parent.s
        self.programname = None

        BaseDialog.__init__(self, "dialogCreateNewKeygroups")

    def set_program(self, programname):
        self.programname = programname
        if type(programname) is list:
            caption_name = ' '.join(programname)
        else:
            caption_name = programname

        self.w_label_create_new.set_label("Create new keygroups on: " + caption_name)

class DialogCreateNewProgramFast(BaseDialog):
    def __init__(self, parent):
        self.s = parent.s
        self.programname = None

        BaseDialog.__init__(self, "dialogCreateNewProgramFast")
        self.w_treeview_selected_samples.append_column(gtk.TreeViewColumn('Name', gtk.CellRendererText(), text=0))

class DialogCreateNewMultiFast(BaseDialog):
    def __init__(self, parent):
        self.s = parent.s

        BaseDialog.__init__(self, "dialogCreateNewMultiFast")
        self.w_treeview_selected_programs.append_column(gtk.TreeViewColumn('Name', gtk.CellRendererText(), text=0))

class BaseContextMenu(base.Base):
    def __init__(self, name, main, tree_view, tools_module):
        base.Base.__init__(self, None, name)
        self.main = main
        self.tree_view = tree_view
        self.tools_module = tools_module
    
    def handle_keyboard_event(self, widget, event):
        if gtk.gdk.keyval_name(event.keyval) == "Delete":
            self.on_delete_activate(None)
        elif gtk.gdk.keyval_name(event.keyval) == "Insert":
          # TODO: create new program/multi
          pass

    def activate_download(self, selected, ext):
        destdir = self.s.FileChooser.open(upload=False, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, title="Select a folder to save all selected files...", multiple=False)
        if destdir is None:
            return

        self.main.log("Downloading file(s) %s to directory '%s'" % (selected, destdir))
        thread.start_new_thread(self._download_and_notify, (destdir, selected, ext))
    
    @transaction()
    def _download_and_notify(self, destdir, selected, ext):
        for f in selected:
            full_name = f + ext
            self.main.s.transfertools.get(full_name, os.path.join(destdir, full_name))

        self.main.log("Download of file(s) %s to '%s' finished" % (selected, destdir))

    def on_delete_activate(self, widget):
        selected = get_selected_from_treeview(self.tree_view)
        for name in selected:
            handle = self.tools_module.get_handle_by_name(name)
            self.main.log("Deleting '%s' with handle %s" % (name, handle))
            self.tools_module.delete_by_handle(handle)

        self.main.init_lists()

    def on_duplicate_activate(self, widget):
        selected = get_selected_from_treeview(self.tree_view)
        for name in selected:
            new_name = 'Copy of ' + name
            self.main.log("Copy '%s' to '%s'" % (name, new_name))
            self.tools_module.set_curr_by_name(name)
            self.tools_module.copy(new_name)
            
        self.main.init_lists()

    def on_edited(self, cell_rend_text, path, new_name):
        name = get_selected_from_treeview(self.tree_view)[0]
        if new_name == name:
            return
        
        self.main.log("Rename '%s' to '%s'" % (name, new_name))
        handle = self.tools_module.get_handle_by_name(name)
        self.tools_module.rename_by_handle(handle, new_name)
            
        self.main.init_lists()

class MultisContextMenu(BaseContextMenu):
    """Context menu for the "multis" TreeView
    """
    def __init__(self, main):
        self.s = main.s
        self.main = main

        BaseContextMenu.__init__(self, "menuMultis", main, main.w_treeview_multis, self.s.multitools)

    def on_multifx_activate(self, widget):
        multi = get_selected_from_treeview(self.main.w_treeview_multis)[0]
        self.s.multitools.set_curr_by_name(multi)
        self.main.open_in_window(multifxeditor.MultiFXEditor(self.s))

    def on_download_activate(self, widget):
        selected = get_selected_from_treeview(self.main.w_treeview_multis)
        self.activate_download(selected, ".akm")
    
    def on_new_multi_activate(self, widget, data=None):
        self.main.on_new_multi_activate(self, widget)
       
class SamplesContextMenu(BaseContextMenu):
    """Context menu for the "samples" TreeView
    """
    def __init__(self, main):
        self.s = main.s
        self.main = main

        BaseContextMenu.__init__(self, "menuSamples", main, main.w_treeview_samples, self.s.sampletools)

        self.dialogCreateNewProgramFast = DialogCreateNewProgramFast(self)
        self.dialogCreateNewProgramFast.w_combo_starting_note.set_model(midiutils.midinotesmodel)
        self.dialogCreateNewProgramFast.w_combo_starting_note.set_active(0)

    def on_download_activate(self, widget):
        selected_samples = get_selected_from_treeview(self.main.w_treeview_samples)
        self.activate_download(selected_samples, ".wav")

    def on_new_program_activate(self, widget):
        selected_samples = get_selected_from_treeview(self.main.w_treeview_samples)
        model = modelutils.get_model_from_list(selected_samples)
        tv = self.dialogCreateNewProgramFast.w_treeview_selected_samples.set_model(model)
        self.dialogCreateNewProgramFast.w_entry_program_name.set_text('Program %i' % (self.main.s.programtools.get_no_items() + 1))
        response = self.dialogCreateNewProgramFast.editor.run()

        if response == gtk.RESPONSE_OK:
            program_name = self.dialogCreateNewProgramFast.w_entry_program_name.get_text()
            # create program
            # figure out note ranges
            # iterate over notes and set up keygroups
            # set keygroup low note and high note and zone 1 sample
            method = self.dialogCreateNewProgramFast.w_combo_allocate_method.get_active()
            starting_note = self.dialogCreateNewProgramFast.w_combo_starting_note.get_active()
            type = self.dialogCreateNewProgramFast.w_combo_program_type.get_active()
            num_samples = len(selected_samples)
            notes = []
            if method == 0:
                # chromatic
                for i in range(starting_note, starting_note+num_samples):
                    notes.append([i, i])
                keytrack = 1
                playback = 0
            elif method == 1:
                # drum
                for i in range(num_samples):
                    notes.append([midiutils.mpcpads[i], midiutils.mpcpads[i]])
                keytrack = 0
                # one shot
                playback = 1
            elif method == 2:
                # span
                for i in range(num_samples):
                    notes.append([0, 127])
                keytrack = 1
                playback = 0

            self.s.programtools.create_new(len(selected_samples), program_name)
            self.s.programtools.set_curr_by_name(program_name)

            if type == 1:
                # drum program
                self.main.log("setting drum type")
                self.s.programtools.set_type(1)

            for i in range(num_samples):
                #self.main.log(str("adding", i, notes[i][0], notes[i][1], selected_samples[i]))
                if type == 0:
                    self.s.keygrouptools.set_curr_keygroup(i)
                    print "set note range"
                    self.s.keygrouptools.set_low_note(notes[i][0])
                    self.s.keygrouptools.set_high_note(notes[i][1])
                else:
                    self.s.keygrouptools.set_curr_keygroup(notes[i][0])
                    
                print "set zone stuff"
                self.s.zonetools.set_keyboard_track(1, keytrack)
                self.s.zonetools.set_playback(1, playback)

                print "set sample"
                self.s.zonetools.set_sample(1, selected_samples[i])
                #self.s.programtools.add_keygroup_sample(notes[i][0],notes[i][1],1,keytrack,selected_samples[i])

            self.main.init_lists()
            self.dialogCreateNewProgramFast.editor.hide()

class ProgramsContextMenu(BaseContextMenu):
    """Context menu for the "programs" TreeView
    """
    def __init__(self, main):
        self.s = main.s
        self.main = main

        BaseContextMenu.__init__(self, "menuPrograms", main, main.w_treeview_programs, main.s.programtools)

        self.dialogCreateNewKeygroups = DialogCreateNewKeygroups(self)
        self.dialogCreateNewMultiFast = DialogCreateNewMultiFast(self)

    def on_add_keygroup_activate(self, widget):
        programname = get_selected_from_treeview(self.main.w_treeview_programs)

        self.dialogCreateNewKeygroups.set_program(programname)

        response = self.dialogCreateNewKeygroups.editor.run()

        if response == gtk.RESPONSE_OK:
            howmany = int(self.dialogCreateNewKeygroups.w_spin_howmany_keygroups.get_value())

            if not (type(programname) is list):
                programname = [programname, ]

            for pn in programname:
                self.main.log("Adding %d keygroups to %s" % (howmany, pn))
                self.program = program.Program(self.s, pn)
                self.program.gettools().add_keygroups(howmany)

    def on_program_properties_activate(self, widget):
        names = get_selected_from_treeview(self.main.w_treeview_programs)
        
	for name in names:
            self.main.open_program_properties(name)

    def on_new_multi_activate(self, widget):
        selected_programs = get_selected_from_treeview(self.main.w_treeview_programs)
        model = modelutils.get_model_from_list(selected_programs)
        self.dialogCreateNewMultiFast.w_treeview_selected_programs.set_model(model)
        self.dialogCreateNewMultiFast.w_entry_name.set_text('Multi %i' % (self.main.s.multitools.get_no_items() + 1))
        response = self.dialogCreateNewMultiFast.editor.run()
        if response == gtk.RESPONSE_OK:
            name = self.dialogCreateNewMultiFast.w_entry_name.get_text()
            self.s.multitools.create_new(len(selected_programs), name)
            self.s.multitools.set_curr_by_name(name)
            for i, p in enumerate(selected_programs):
                self.s.multitools.set_multi_part_name(i, p)
            self.main.init_lists()

    def on_recycle_init_activate(self, widget):
        programname = get_selected_from_treeview(self.main.w_treeview_programs)
        if not (type(programname) is list):
            programname = [programname, ]
        
        for pn in programname:
            p = program.Program(self.s, pn)
            p.init_recycled()

    def on_download_activate(self, widget):
        selected = get_selected_from_treeview(self.main.w_treeview_programs)
        self.activate_download(selected, ".akp")
            
    def on_dump_matrix(self, widget):
        programname = get_selected_from_treeview(self.main.w_treeview_programs)
        if not (type(programname) is list):
            programname = [programname, ]

        for pn in programname:
            p = program.Program(self.s, pn)
            matrix = p.dump_matrix()
            self.main.log(matrix)

    def on_keygroup_editor_activate(self, widget):
        programname = get_selected_from_treeview(self.main.w_treeview_programs)
        # multiple selection is possible, but for now we'll just take the first one 
        programname = programname[0]
        
        self.main.open_keygroup_editor(programname)
        
    def on_new_program_activate(self, widget):
        self.main.on_new_program_activate(widget)
                   
    def on_set_current_program_activate(self, widget):
        print "set current program"


class Main(base.Base):
    """Main Window
    """
    def __init__(self, s):
        self.dnd_list = [ ('text/uri-list', 0, TARGET_TYPE_URI_LIST) ] 
        self.s = s
        self.kgeditwindow = None
        self.multieditwindow = None
        self.program_details_window = None
        base.Base.__init__(self, None, "vboxMain")

        setattr(self.s, 'FileChooser', filechooser.FileChooser(s))
        
        self.ProgramsContextMenu = ProgramsContextMenu(self)
        self.SamplesContextMenu = SamplesContextMenu(self)
        self.MultisContextMenu = MultisContextMenu(self) 

        self.configure_treeview(self.w_treeview_programs, self.ProgramsContextMenu)
        self.configure_treeview(self.w_treeview_samples, self.SamplesContextMenu)
        self.configure_treeview(self.w_treeview_multis, self.MultisContextMenu)

        self.w_quit1.connect('activate', gtk.main_quit)
        
        vadj = self.w_console_window.get_vadjustment()
        vadj.connect('changed', lambda a, s=self.w_console_window: self.rescroll(a, s))
        
        self.init_lists()

        """
        self.programsEditor = programseditor.ProgramsEditor(self.s)
        """
        self.record = recorddialog.RecordDialog(recording.Recording(self.s))

        self.on_update_models(None)
        
    def configure_treeview(self, tv, context_menu):
        text = gtk.CellRendererText()
        text.set_property("editable", True)
        text.connect("edited", context_menu.on_edited)
        tv.append_column(gtk.TreeViewColumn("Name", text, text=0))
        tv.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        tv.connect("drag_data_received", self.on_drag_data_received)
        tv.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP, self.dnd_list, gtk.gdk.ACTION_COPY)

        tv.add_events(gtk.gdk.KEY_PRESS)
        tv.connect("key_press_event", context_menu.handle_keyboard_event)
      
    @staticmethod
    def get_names(module):
        handles_names = module.get_handles_names()
        names = []
        for i in range(0, len(handles_names), 2):
            names.append(handles_names[i+1])
        return names
        
    @staticmethod
    def do_programlist(s):
        setattr(s, 'programs', Main.get_names(s.programtools))
        setattr(s, 'programsmodel', modelutils.get_model_from_list(s.programs))
        
    @staticmethod
    def do_lists(s):
        Main.do_programlist(s)
        
        setattr(s, 'samples', Main.get_names(s.sampletools))
        setattr(s, 'samplesmodel', modelutils.get_model_from_list(s.samples))
        
        setattr(s, 'multis', Main.get_names(s.multitools))
        setattr(s, 'multismodel', modelutils.get_model_from_list(s.multis))

    def on_drag_data_received(self, widget, context, x, y, selection, target_type, timestamp):
        self.s.FileChooser.on_drag_data_received(widget, context, x, y, selection, target_type, timestamp)
        self.init_lists()
                
    def set_window(self, window):
        self.window = window
        self.window.set_title("aksui %s" % (__version__))
        self.window.connect('configure_event', self.on_configure_event)
        self.window.connect("drag_data_received", self.on_drag_data_received)
        self.window.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP, self.dnd_list, gtk.gdk.ACTION_COPY)

    def log(self, text):
        self.w_console.get_buffer().insert_at_cursor(text + "\r\n")

    def rescroll(self, vadj, scroll):
        vadj.set_value(vadj.upper-vadj.page_size)
        scroll.set_vadjustment(vadj)        
        
    def move_properties_window(self):
        position = self.window.get_position()
        size = self.window.get_size()
        decoration_width = 10
        if self.program_details_window is not None:
            self.program_details_window.editor.move(position[0] + size[0] + decoration_width, position[1])
        
    def init_lists(self):
        try:
            Main.do_lists(self.s)
            self.s.samplesmodel.connect("row-changed", self.on_update_models)
            self.s.programsmodel.connect("row-changed", self.on_update_models)
            self.s.multismodel.connect("row-changed", self.on_update_models)

            self.on_update_models(None)
            self.log("Multis, Programs, and Samples Loaded...")
        except Exception, ex:
            self.log("Exception: %s" % (ex))
            
    def open_multi_editor(self, multiname):
        if multiname:
            m = multi.Multi(self.s, multiname)
            if not self.multieditwindow:
                self.multieditwindow = multieditor.MultiEditorWindow(self.s, m)
            else:
                self.multieditwindow.setup(m)
            self.multieditwindow.show_all()
                
    def open_keygroup_editor(self, programname):
        if programname:
            if not self.kgeditwindow or self.kgeditwindow.p.name != programname:
                p = program.Program(self.s, programname)
                if not self.kgeditwindow:
                    self.kgeditwindow = keygroupeditor.KeygroupEditorWindow(self.s, p)
                else:
                    self.kgeditwindow.setup(p)
            self.kgeditwindow.show_all()
                
    def on_refresh_clicked(self, widget):
        self.init_lists()

    def on_update_program_model(self):
        self.w_treeview_programs.set_model(self.s.programsmodel)
        
    def on_update_multi_model(self):
        self.w_treeview_multis.set_model(self.s.multismodel)
        
    def on_update_models(self, model, iter = None, user_param = None):
        self.on_update_program_model()
        
        self.on_update_multi_model()
        
        self.w_treeview_samples.set_model(self.s.samplesmodel)

    def get_curr_programname(self):
        selection = self.w_treeview_programs.get_selection()

        model, iter = selection.get_selected()
        programname = model[iter][0]
        return programname

    def open_program_properties(self, programname):
        p = program.Program(self.s, programname)
        
        if not self.program_details_window:
            self.program_details_window = programdetails.ProgramDetails(p)
        else:
            self.program_details_window.set_samplerobject(p)
        
        self.move_properties_window()
        self.program_details_window.show_all()
        pass
    
    def on_recording_activate(self, button):
        self.log("record activate")
        self.record.show_all()
        
    def on_program_editor_activate(self, button):
        self.programsEditor.programsMain.show_all()

    def create(self, tools, template, no_items):
        i = tools.get_no_items() + 1
        name = template % i
        tools.create_new(no_items, name)
        self.log("%s created" % name)
        
    def on_new_program_activate(self, widget, data=None):
        self.create(self.s.programtools, "Program %i", 4)
        self.init_lists()
                
    def on_new_multi_activate(self, widget, data=None):
        self.create(self.s.multitools, "Multi %i", 12)
        self.init_lists()
        
    def on_save_activate(self, button):
        # THIS WILL OVERWRITE FILES w/ SAME NAMES!
        # get folder to save to
        path = self.s.FileChooser.open(upload=False, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, title="Save all files...", multiple=False)
            
        if path:
            org = {'multitools':'.akm', 'programtools':'.akp', 'sampletools' : '.wav'}
            results = []
            for toolname in org.keys():
                ext = org[toolname]
                tool = getattr(self.s, toolname)
                items = tool.get_names()
                if type(items) is str:
                    items = [items, ]

                for item in items:
                    if len(item) > 0:
                        filename = item + ext
                        filenamepath = path + "/" + filename
                        if os.path.exists(filenamepath):
                            # TODO: Put some sort of confirmation here, if user wants it.
                            self.log(filenamepath + " exists; overwriting it.")
                        self.log("Saving " + filenamepath + "...")
                        self.s.transfertools.get(filename, filenamepath)
        else:
            self.log("Invalid path chosen.")
            
    def on_upload_activate(self, button):
        self.s.FileChooser.open(upload=True)
        self.init_lists()

    def on_about_activate(self, button):
        dialog = gtk.AboutDialog()
        dialog.set_name('aksui')
        dialog.set_version(__version__)
        dialog.set_authors((__author__,))
        dialog.set_license(__license__)
        dialog.set_comments("Take control of your sampler!")
        dialog.set_website("http://walco.n--tree.net/projects/aksy/")
        dialog.connect('response', lambda dialog, data: dialog.destroy())
        dialog.show_all()
        
    def on_configure_event(self, widget, event):
        self.move_properties_window()
        return False

    def on_lcd_activate(self, button):
        self.open_in_window(lcdscreen.LCDScreen(self.s))

    def open_in_window(self, widget):
        win = gtk.Window()
        win.add(widget)
        win.show_all()

    def on_treeview_event(self, widget, event):
        """Handles context menus + doubleclicks.
        """

        if widget == self.w_treeview_programs:
            if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
                self.ProgramsContextMenu.editor.popup(None, None, None, event.button, event.time)
                return True

            if event.type == gtk.gdk._2BUTTON_PRESS:
                curr_programs = get_selected_from_treeview(self.w_treeview_programs)
                if len(curr_programs) == 0:
                    return False

                self.open_keygroup_editor(curr_programs[0])
                
	"""
	OLD ONE:
	self.programsEditor.set_program(curr_program)
	self.programsEditor.programsMain.show_all()
	"""

        if widget == self.w_treeview_multis:
            if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
                self.MultisContextMenu.editor.popup(None, None, None, event.button, event.time)
                return True

            if event.type == gtk.gdk._2BUTTON_PRESS:
                curr_multi = get_selected_from_treeview(self.w_treeview_multis)
                if len(curr_multi) == 0:
                    return False
                
                curr_multi = curr_multi[0]

                self.open_multi_editor(curr_multi)
                
        if widget == self.w_treeview_samples:
            if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
                self.SamplesContextMenu.editor.popup(None, None, None, event.button, event.time)
                return True

log = None

if USE_CUSTOM_EXCEPTHOOK:
    import sys
    sys.excepthook = exceptionHandler

def add_keybinding(accel_group, widget, accel_str, signal="activate"):
    keyval, mods = gtk.accelerator_parse(accel_str)
    widget.add_accelerator(signal, accel_group, 
                                       keyval, mods, gtk.ACCEL_VISIBLE)

    
def setup_keybindings(m, accel_group):
    add_keybinding(accel_group, m.w_quit1, "<Ctl>Q")
    add_keybinding(accel_group, m.w_refresh_button, "F5", signal="clicked")
 
def main(): 
    parser = config.create_option_parser(usage="%prog [options]")
    options = parser.parse_args()[0]

    sampler = Devices.get_instance(options.sampler_type, options.connector)

    try:
        m = Main(sampler)
        accel_group = gtk.AccelGroup()
        
        win = gtk.Window()
        win.add_accel_group(accel_group)
        setup_keybindings(m, accel_group)

        win.add(m.editor)

        m.set_window(win)
        win.show_all()
        # TODO: make edit menu functional
        m.w_edit_menu.hide()
        win.connect("delete-event", gtk.main_quit)

        gtk.main()
    finally:
        sampler.close()

if __name__ == "__main__":
    if ENABLE_PROFILER:
        prof = hotshot.Profile("ak.py.prof")
        prof.runcall(main)
        stats = hotshot.stats.load("ak.py.prof")
        stats.sort_stats('time', 'calls')
        stats.print_stats()
    else:
        main()



import gtk

import panelbase, rangewidget
from aksui.utils import midiutils
from aksui.ak import lfo

class LFOPanel(panelbase.PanelBase):
    def __init__(self,kg,cb=None):
        panelbase.PanelBase.__init__(self,kg,"LFOs",cb)

    def setup(self, kg):
        self.clear_children(True)

        self.kg = kg
        self.s = kg.s
        lfovbox = gtk.VBox()
        for i in range(2):
            lfohbox = gtk.HBox()
            l = lfo.LFO(kg.s, kg, i)
            # two lfos
            lfohbox.pack_start(rangewidget.AkKnobWidget(l, "lfo_depth", 0, 100, 1, None), False, False, 0)
            lfohbox.pack_start(rangewidget.AkKnobWidget(l, "lfo_rate", 0, 100, 1, None), False, False, 0) # what is this in hz?
            lfohbox.pack_start(rangewidget.AkKnobWidget(l, "lfo_delay", 0, 100, 1, None), False, False, 0) # what is this in ms?
            lfohbox.pack_start(rangewidget.AkKnobWidget(l, "lfo_phase", 0, 360, 1, None), False, False, 0)
            lfohbox.pack_start(rangewidget.AkKnobWidget(l, "lfo_shift", -50, 50, 1, None), False, False, 0)
            lfohbox.pack_start(rangewidget.AkComboBox(l, "lfo_waveform", midiutils.sampler_lists["lfo_waves"]), True, True, 0)
            lfohbox.pack_start(rangewidget.AkComboBox(l, "lfo_retrigger", midiutils.sampler_lists["lfo_retrigger"]), True, True, 0)
            lfohbox.pack_start(rangewidget.AkComboBox(l, "lfo_sync", midiutils.sampler_lists["lfo_sync"]), True, True, 0)
            lfohbox.pack_start(rangewidget.AkComboBox(l, "lfo_midi_sync", midiutils.sampler_lists["lfo_midi_sync"]), True, True, 0)
            lfovbox.pack_start(lfohbox)
        self.pack_start(lfovbox, False, False, 1)
        self.show_all()




""" Python equivalent of akai section sysextools

Methods to manipulate sysex parameters
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Sysextools:
    def __init__(self, z48):
        self.sampler = z48
        self.query_cmd = Command('_', '\x00\x00', 'sysextools', 'query', (), None)
        self.enable_msg_notification_cmd = Command('_', '\x00\x01', 'sysextools', 'enable_msg_notification', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.enable_item_sync_cmd = Command('_', '\x00\x03', 'sysextools', 'enable_item_sync', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.enable_checksum_verification_cmd = Command('_', '\x00\x04', 'sysextools', 'enable_checksum_verification', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.enable_screen_updates_cmd = Command('_', '\x00\x05', 'sysextools', 'enable_screen_updates', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.echo_cmd = Command('_', '\x00\x06', 'sysextools', 'echo', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.enable_heartbeat_cmd = Command('_', '\x00\x07', 'sysextools', 'enable_heartbeat', (), None)
        self.enable_playback_sync_cmd = Command('_', '\x00\x08', 'sysextools', 'enable_playback_sync', (aksy.devices.akai.sysex_types.BOOL,), None)
        self.get_sysex_buffersize_cmd = Command('_', '\x00\x10', 'sysextools', 'get_sysex_buffersize', (), None)

    def query(self):
        """Query device, returns device ID
        """
        return self.sampler.execute(self.query_cmd, ())

    def enable_msg_notification(self, arg0):
        """Enable/disable 'OK' messages. Use with care!
        """
        return self.sampler.execute(self.enable_msg_notification_cmd, (arg0, ))

    def enable_item_sync(self, arg0):
        """Enable sync between sysex item and front panel
        """
        return self.sampler.execute(self.enable_item_sync_cmd, (arg0, ))

    def enable_checksum_verification(self, arg0):
        """Enable sysex msg checksum verifcation
        """
        return self.sampler.execute(self.enable_checksum_verification_cmd, (arg0, ))

    def enable_screen_updates(self, arg0):
        """Enable screen updates during sysex processing
        """
        return self.sampler.execute(self.enable_screen_updates_cmd, (arg0, ))

    def echo(self, arg0, arg1, arg2, arg3):
        """Echo the specified bytes

        Returns:
            FOUR_BYTES
        """
        return self.sampler.execute(self.echo_cmd, (arg0, arg1, arg2, arg3, ))

    def enable_heartbeat(self):
        """Enable sending of empty sysex 'still alive' msgs
        """
        return self.sampler.execute(self.enable_heartbeat_cmd, ())

    def enable_playback_sync(self, arg0):
        """Enable sync between sysex playback item and front panel
        """
        return self.sampler.execute(self.enable_playback_sync_cmd, (arg0, ))

    def get_sysex_buffersize(self):
        """Retrieves the sysex buffer size

        Returns:
            WORD
        """
        return self.sampler.execute(self.get_sysex_buffersize_cmd, ())




# -*- coding: latin-1 -*-
import os,os.path

from aksy.device import Devices
from aksy import config
from aksy.devices.akai.sampler import Sampler
from aksy.devices.akai import fileparser
import aksy.fileutils

__author__ = 'Joseph Misra/Walco van Loon'
__version__ = '0.01'

def create_option_parser(): 
    usage = "%prog [files|folders]"
    parser = config.create_option_parser(usage)
    parser.add_option("-p", nargs=1, dest="programName",
          help="Upload the specified program and the samples it references")
    parser.add_option("-d", nargs=1, dest="drumProgramName",
          help="Create and upload a new drum program from files using a specific naming scheme")
    return parser

def collect_files(args):
    collected = []
    for f in args:
        if os.path.isfile(f):
            if Sampler.is_filetype_supported(f):
                collected.append(f)
        elif os.path.isdir(f):
            collected.extend(collect_dir(f))
        else:
            raise IOError("File not found: " + repr(f))
    return collected

def collect_dir(args):
    for root, dir, files in os.walk(args):
        for found in files:
            if Sampler.is_filetype_supported(found):
                yield os.path.join(root, found)
                  
def main():
    parser = create_option_parser()
    (options, filelist) = parser.parse_args()

    if len(filelist) == 0:
        parser.error("At least one or more files/directories should be specified")
  
    to_upload = collect_files(filelist)
    
    if len(to_upload) == 0:
        parser.error("Nothing to upload: no supported files found")
    
    z48 = Devices.get_instance(options.sampler_type, options.connector)    
    
    if options.programName is not None:
        print 'Uploading files.'
        upload_files(z48, to_upload, filterProgram=options.programName)
    elif options.drumProgramName:
        print 'Uploading files.'
        print 'Building drum program.', options.drumProgramName
        build_drum_program(z48, to_upload, options.drumProgramName)
        upload_files(z48, to_upload, filterProgram=options.drumProgramName)
    else:
        upload_files(z48, to_upload)

def build_drum_program(z48, filelist, program_name):
    # create drum program named program_name
    # iterate through arguments and set each sample name to its key
    bankmap = {"A":0, "B":1, "C":2, "D":3}
    keymap = []
    for file in filelist:
        base, name = os.path.splitext(file)
        bank = bankmap[name[0]]
        bankindex = int(name[1:]) - 1
        midinote = mpcpads[(bank * 16) + bankindex]
        keymap.append([name, midinote])
        
    programs = z48.programtools.get_names()
    if not program_name in programs:
        z48.programtools.create_new(1, program_name)
    z48.programtools.set_curr_by_name(program_name)
    z48.programtools.set_type(1) # should make 128 keygroups?
    
    p = program(z48,program_name)
    
    for name, midinote in keymap:
        kg = keygroup(p, midinote)
        print midinote, name
        kg.zones[0].set("sample", program_name + " " + name)
        kg.zones[0].set("playback", 1)
        kg.set("polyphony", 1)
        
    # set hihat mute group
    kg = keygroup(p, 46)
    kg.set("mute_group", 1)
    kg = keygroup(p, 42)
    kg.set("mute_group", 1)
    # set this on program
    
def upload_files(z48, to_upload, filterProgram=None):
    if filterProgram is not None:
        filtered = []
        program = fileparser.ProgramParser().parse(filterProgram)
        for kg in program.keygroups:
            for zone in kg.zones:
                if zone.samplename:
                    filtered.append(find_file(to_upload, zone.samplename))
        to_upload = filtered
    
    for file in to_upload:
        z48.transfertools.put(file)

def find_file(files, samplename):
    for f in files:
        filename = os.path.basename(f).lower()
        basename, ext = os.path.splitext(filename)
        if basename == samplename:
            return f
    raise IOError("File not found in upload file list: " + samplename)



import gtk

class HitBox:
    def __init__(self,x,y,w,h):
        self.x = x
        self.y = y 
        self.w = w 
        self.h = h
        self.is_scaled = False
        self.index = 0

    def point_in(self,x,y):
        horiz = False
        vert = False

        if x >= self.x and x <= self.x+self.w:
            horiz = True
        if y >= self.y and y <= self.y+self.h:
            vert = True

        if horiz and vert:
            return True
        else:
            return False

    def get_scale(self):
        if self.widget and self.is_scaled:
            windowsize = self.widget.window.get_size()
            buffer = (self.w * 4) + 60
            windowsize = [windowsize[0] - buffer, windowsize[1]]
            range = self.max - self.min
            scale = float(range) / float(windowsize[0]) # get percentage of window width the control has
            return scale
        else:
            return 1.0

    def get_scaled_value(self):
        return self.x * 1/self.get_scale()

    def get_offset(self):
        if self.index == 0:
            offset = -self.w / 2
        elif self.index == 1:
            offset = (self.w / 2) + 1
        else:
            offset = 0

        return offset


    def get_scaled_rect(self):
        scale = self.get_scale()
        if scale > 0:
            rect = self.get_rectangle()

            x = int(rect.x * (1/scale))

            offset = self.get_offset()

            rect = gtk.gdk.Rectangle(x + offset, rect.y, rect.width, self.h)
            return rect
        else:
            return None

    def get_widget_rect(self, widget = None):
        # set widget it if was given, otherwise leave it
        if widget:
            self.widget = widget

        if self.widget:
            rect = self.get_scaled_rect()
            win = self.widget.window
            size = win.get_size()
            vertical_align = (size[1] / 2) - rect.height / 2

            if self.is_scaled:
                rect.x += rect.width
                
            return gtk.gdk.Rectangle(int(rect.x), int(vertical_align), int(rect.width), int(rect.height))
        else:
            return None

    def point_in_widget_rect(self, x, y):
        #print self.x, ',', self.y, ' ', x, ',', y
        rect = self.get_widget_rect()

        horiz = False
        vert = False

        if x >= rect.x and x <= rect.x+rect.width:
            horiz = True
        if y >= rect.y and y <= rect.y+rect.height:
            vert = True

        if horiz and vert:
            return True
        else:
            return False

    def get_centered_rect(self):
        # get centered rectangle for drawing
        r = self.get_rectangle()
        r = gtk.gdk.Rectangle(int(self.x - (r.x - r.width/2)), int(self.y - (r.y - r.height / 2)), int(self.w), int(self.h))
        return r

    def get_rectangle(self):
        return gtk.gdk.Rectangle(int(self.x), int(self.y), int(self.w), int(self.h)) 




from aksyosc.osc import CallbackManager, decodeOSC, OSCMessage

from aksy.concurrent import transaction

import logging

LOG = logging.getLogger('aksy.osc')

class DispatchException(Exception):
    pass

class SamplerCallbackManager(CallbackManager):
    def __init__(self, sampler):
        CallbackManager.__init__(self)
        self.sampler = sampler

    def handle(self, data, source = None):
        """Given OSC data, tries to call the callback with the
        right address, and returns the result"""
        decoded = decodeOSC(data)
        return self.dispatch(decoded)

    @transaction()
    def dispatch(self, message):
        """
            Overrides base class method to return the result of the dispatch.
        """
        if LOG.isEnabledFor(logging.DEBUG):
            LOG.debug('dispatch(%s)', repr(message))

        address = message[0]
            
        # skip address, typetag
        try:
            if address == "#bundle":
                return self.dispatch_alt_request(message[2:])
            else:
                return self.dispatch_command(address, message)
        except AttributeError, e:
            LOG.exception("OSC message %s could not be dispatched", repr(message))
            return create_error_msg("Failed to execute command %s" % address, e)
        except Exception, e:
            LOG.exception("Dispatch of %s failed", repr(message))
            return create_error_msg('Execution failed', e)

    def parse_cmd_name(self, address):
        comps = address.split('/')
        if len(comps) != 3:
            raise DispatchException("Invalid address: '%s', should have two components" % address)
        return comps[1:3]
    
    def dispatch_command(self, address, message):
        section, cmd_name = self.parse_cmd_name(address)
        result = self.sampler.execute_by_cmd_name(section, cmd_name, message[2:])
        return create_response_msg(address, result)

    def dispatch_alt_request(self, messages):
        alt_request = messages.pop(0)
        alt_address = alt_request[0]

        handle = alt_request[2]
        index = alt_request[3]

        cmds = []
        args = []
        for msg in messages:
            address = msg[0]
            args.append(msg[2:])
            section, cmd_name = self.parse_cmd_name(address)
            cmds.append(self.sampler.get_cmd_by_name(section, cmd_name))

        result = self.sampler.execute_alt_request(handle, cmds, args, index)
        return create_response_msg(alt_address, result)
        
def create_response_msg(address, result):
    msg = OSCMessage()
    msg.setAddress(address)
    if not hasattr(result, '__iter__'):
        result = [result]

    for arg in result:
        msg.append(arg)
    return msg.getBinary()

def create_error_msg(text, exception):
    msg = OSCMessage()
    msg.setAddress('/sampler/error')
    msg.append(text)
    msg.append(str(exception))
    return msg.getBinary()




""" Python equivalent of akai section zonetools

Manipulate the zones of a keygroup
"""

__author__ =  'Walco van Loon'
__version__ =  '0.2'

from aksy.devices.akai.sysex import Command

import aksy.devices.akai.sysex_types

class Zonetools:
    def __init__(self, z48):
        self.sampler = z48
        self.get_sample_cmd = Command('_', '\x0F\x01', 'zonetools', 'get_sample', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_level_cmd = Command('_', '\x0F\x02', 'zonetools', 'get_level', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_pan_cmd = Command('_', '\x0F\x03', 'zonetools', 'get_pan', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_output_cmd = Command('_', '\x0F\x04', 'zonetools', 'get_output', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_filter_cmd = Command('_', '\x0F\x05', 'zonetools', 'get_filter', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_tune_cmd = Command('_', '\x0F\x06', 'zonetools', 'get_tune', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_keyboard_track_cmd = Command('_', '\x0F\x07', 'zonetools', 'get_keyboard_track', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_playback_cmd = Command('_', '\x0F\x08', 'zonetools', 'get_playback', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_mod_start_cmd = Command('_', '\x0F\x09', 'zonetools', 'get_mod_start', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_low_velocity_cmd = Command('_', '\x0F\x0A', 'zonetools', 'get_low_velocity', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_high_velocity_cmd = Command('_', '\x0F\x0B', 'zonetools', 'get_high_velocity', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_mute_cmd = Command('_', '\x0F\x0C', 'zonetools', 'get_mute', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.get_solo_cmd = Command('_', '\x0F\x0D', 'zonetools', 'get_solo', (aksy.devices.akai.sysex_types.BYTE,), None)
        self.set_sample_cmd = Command('_', '\x0E\x01', 'zonetools', 'set_sample', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.STRING), None)
        self.set_level_cmd = Command('_', '\x0E\x02', 'zonetools', 'set_level', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SWORD), None)
        self.set_pan_cmd = Command('_', '\x0E\x03', 'zonetools', 'set_pan', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_output_cmd = Command('_', '\x0E\x04', 'zonetools', 'set_output', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_filter_cmd = Command('_', '\x0E\x05', 'zonetools', 'set_filter', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SBYTE), None)
        self.set_tune_cmd = Command('_', '\x0E\x06', 'zonetools', 'set_tune', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SWORD), None)
        self.set_keyboard_track_cmd = Command('_', '\x0E\x07', 'zonetools', 'set_keyboard_track', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BOOL), None)
        self.set_playback_cmd = Command('_', '\x0E\x08', 'zonetools', 'set_playback', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_mod_start_cmd = Command('_', '\x0E\x09', 'zonetools', 'set_mod_start', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.SWORD), None)
        self.set_low_vel_cmd = Command('_', '\x0E\x0A', 'zonetools', 'set_low_vel', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_high_vel_cmd = Command('_', '\x0E\x0B', 'zonetools', 'set_high_vel', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_mute_cmd = Command('_', '\x0E\x0C', 'zonetools', 'set_mute', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)
        self.set_solo_cmd = Command('_', '\x0E\x0D', 'zonetools', 'set_solo', (aksy.devices.akai.sysex_types.BYTE, aksy.devices.akai.sysex_types.BYTE), None)

    def get_sample(self, arg0):
        """Get Zone Sample (Data1=zone number 1-4)

        Returns:
            STRING
        """
        return self.sampler.execute(self.get_sample_cmd, (arg0, ))

    def get_level(self, arg0):
        """Get Zone Level <Reply> = level in 10 dB

        Returns:
            LEVEL
        """
        return self.sampler.execute(self.get_level_cmd, (arg0, ))

    def get_pan(self, arg0):
        """Get Zone Pan (0-100)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_pan_cmd, (arg0, ))

    def get_output(self, arg0):
        """Get Zone Output(0-15)

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_output_cmd, (arg0, ))

    def get_filter(self, arg0):
        """Get Zone Filter (0 +-100)

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_filter_cmd, (arg0, ))

    def get_tune(self, arg0):
        """Get Zone Cents Tune(0 +-3600)

        Returns:
            SBYTE
        """
        return self.sampler.execute(self.get_tune_cmd, (arg0, ))

    def get_keyboard_track(self, arg0):
        """Get Zone Keyboard Track

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_keyboard_track_cmd, (arg0, ))

    def get_playback(self, arg0):
        """Get Zone Playback

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_playback_cmd, (arg0, ))

    def get_mod_start(self, arg0):
        """Get Zone ModStart(0 +- 9999)

        Returns:
            SWORD
        """
        return self.sampler.execute(self.get_mod_start_cmd, (arg0, ))

    def get_low_velocity(self, arg0):
        """Get Zone Low Velocity

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_low_velocity_cmd, (arg0, ))

    def get_high_velocity(self, arg0):
        """Get Zone High Velocity

        Returns:
            BYTE
        """
        return self.sampler.execute(self.get_high_velocity_cmd, (arg0, ))

    def get_mute(self, arg0):
        """Get Zone Mute

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_mute_cmd, (arg0, ))

    def get_solo(self, arg0):
        """Get Zone Solo

        Returns:
            BOOL
        """
        return self.sampler.execute(self.get_solo_cmd, (arg0, ))

    def set_sample(self, arg0, arg1):
        """Set Zone Sample <Data2...0> = name of sample to assign to zone. (0, 1-4)
        """
        return self.sampler.execute(self.set_sample_cmd, (arg0, arg1, ))

    def set_level(self, arg0, arg1):
        """Set Zone Level <Data1> = Zone number, <Data2> = level in 10xdB (0, 1-4)
        """
        return self.sampler.execute(self.set_level_cmd, (arg0, arg1, ))

    def set_pan(self, arg0, arg1):
        """Set Zone Pan/Balance <Data2> = Pan/Bal where (0-100 = L50-R50);
        """
        return self.sampler.execute(self.set_pan_cmd, (arg0, arg1, ))

    def set_output(self, arg0, arg1):
        """Set Zone Output <Data2> = output, where 0=MULTI, 1 = L/R; 2-5 = op1/2;op7/8; 6-15 = L, R, op1-op8
        """
        return self.sampler.execute(self.set_output_cmd, (arg0, arg1, ))

    def set_filter(self, arg0, arg1):
        """Set Zone Filter
        """
        return self.sampler.execute(self.set_filter_cmd, (arg0, arg1, ))

    def set_tune(self, arg0, arg1):
        """Set Zone Cents Tune
        """
        return self.sampler.execute(self.set_tune_cmd, (arg0, arg1, ))

    def set_keyboard_track(self, arg0, arg1):
        """Set Zone Keyboard Track
        """
        return self.sampler.execute(self.set_keyboard_track_cmd, (arg0, arg1, ))

    def set_playback(self, arg0, arg1):
        """Set Zone Playback <Data2> = mode, where 0=NO LOOPING, 1=ONE SHOT 2=LOOP IN REL, 3=LOOP UNTIL REL, 4=LIRRETRIG, 5=PLAYRETRIG, 6=AS SAMPLE
        """
        return self.sampler.execute(self.set_playback_cmd, (arg0, arg1, ))

    def set_mod_start(self, arg0, arg1):
        """Set Zone ModStart
        """
        return self.sampler.execute(self.set_mod_start_cmd, (arg0, arg1, ))

    def set_low_vel(self, arg0, arg1):
        """Set Zone Low Velocity
        """
        return self.sampler.execute(self.set_low_vel_cmd, (arg0, arg1, ))

    def set_high_vel(self, arg0, arg1):
        """Set Zone High Velocity
        """
        return self.sampler.execute(self.set_high_vel_cmd, (arg0, arg1, ))

    def set_mute(self, arg0, arg1):
        """Set Zone Mute <Data2> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_mute_cmd, (arg0, arg1, ))

    def set_solo(self, arg0, arg1):
        """Set Zone Solo <Data2> = (0=OFF, 1=ON)
        """
        return self.sampler.execute(self.set_solo_cmd, (arg0, arg1, ))

